----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 16.04.2019 19:00:01
-- Design Name: 
-- Module Name: Frame_Generator - struct
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Frame_Generator is
    Port ( mLengthFG : in STD_LOGIC_VECTOR (7 downto 0);
           mDataFG : in STD_LOGIC_VECTOR (7 downto 0);
           mValidFG : in STD_LOGIC;
           mLastFG : in STD_LOGIC;
           eGenFG : in STD_LOGIC;
           pGenFG : in STD_LOGIC;
           mLengthTotalFG : in STD_LOGIC_VECTOR (7 downto 0);
           enableFG : in STD_LOGIC;
           rstFG : in STD_LOGIC;
           clkFG : in STD_LOGIC;
           dataOutFG : out STD_LOGIC_VECTOR (7 downto 0);
           readyInFG : in STD_LOGIC;
           validOutFG : out STD_LOGIC;
           clkOutFG : out STD_LOGIC;
           header_genFG : out STD_LOGIC;
           data_genFG : out STD_LOGIC;
           eTxOutFG : out STD_LOGIC;
           pTxOutFG : out STD_LOGIC;
           rstGenFG : out STD_LOGIC);
end Frame_Generator;

architecture struct of Frame_Generator is

    -- Componentes
    component preamble_generator 
       Port ( clk : in STD_LOGIC;
              enable_generate : in STD_LOGIC;
              pInterrupt : in STD_LOGIC;
              eGen : in STD_LOGIC;
              pGen : in STD_LOGIC;
              preamble : out STD_LOGIC_VECTOR (7 downto 0);
              ended : out STD_LOGIC;
              valid : out STD_LOGIC);
    end component;
    
    component payload_data_generator 
        Port ( clk : in STD_LOGIC;
               enable_generate : in STD_LOGIC;
               mData : in STD_LOGIC_VECTOR (7 downto 0);
               mLast : in STD_LOGIC;
               payload : out STD_LOGIC_VECTOR (7 downto 0);
               ended_header : out STD_LOGIC;
               ended_data : out STD_LOGIC);
    end component;
    
    component select_generated_data is
        Port (  clk : in STD_LOGIC;
                preamble : in STD_LOGIC_VECTOR (7 downto 0);
                payload : in STD_LOGIC_VECTOR (7 downto 0);
                fsc : in STD_LOGIC_VECTOR (7 downto 0);
                sel : in STD_LOGIC_VECTOR (3 downto 0);
                data : out STD_LOGIC_VECTOR (7 downto 0));
    end component;
    
    component select_generating_block is
        Port ( clk : in STD_LOGIC;
               rst : in STD_LOGIC;
               mGen : in STD_LOGIC;
               endBlock : in STD_LOGIC_VECTOR (4 downto 0);
               pInterrupt : in STD_LOGIC;
               header_gen : out STD_LOGIC;
               data_gen : out STD_LOGIC;
               selBlock : out STD_LOGIC_VECTOR (3 downto 0);
               fin_gen : out STD_LOGIC;
               valid : out STD_LOGIC);
    end component;
    
   component fsc_generator is
        Port ( clk : in STD_LOGIC;
               enable : in STD_LOGIC;
               mData : in STD_LOGIC_VECTOR (7 downto 0);
               mLast : in STD_LOGIC;
               --end_data_gen : in STD_LOGIC;
               data_gen : in STD_LOGIC;
               pInterrupt : in STD_LOGIC;
               fsc : out STD_LOGIC_VECTOR (7 downto 0);
               ended : out STD_LOGIC);
    end component;
    
    component frame_generator_control is
        Port ( enable : in STD_LOGIC;
               eGen : in STD_LOGIC;
               pGen : in STD_LOGIC;
               mValid : in STD_LOGIC;
               mLast : in STD_LOGIC;
               rst : in STD_LOGIC;
               clk : in STD_LOGIC;
               fin_gen : in STD_LOGIC;
               mGen : out STD_LOGIC;
               eTx : out STD_LOGIC;
               pTx : out STD_LOGIC;
               rstGen : out STD_LOGIC;
               pInterrupt: out STD_LOGIC;
               ready : in STD_LOGIC);
    end component;
        -------------------------------------------
        -- Seales interconexion de bloques
        -------------------------------------------
        
        -------------------------------------------
        --General 
        -------------------------------------------
        --Inputs
        signal mLength_sig :  STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal mData_sig :  STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal mValid_sig :  STD_LOGIC := '0';
        signal mLast_sig :  STD_LOGIC := '0';
        signal eGen_sig :  STD_LOGIC := '0';
        signal pGen_sig :  STD_LOGIC := '0';
        signal mLengthTotal_sig :  STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal enable_sig :  STD_LOGIC := '0';
        signal rst_sig :  STD_LOGIC := '0';
        signal clk_sig :  STD_LOGIC := '0';
        signal readyIn_sig :  STD_LOGIC := '0';   
        --Outputs
        signal dataOut_sig :  STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal dataOut_retrasado1_sig :  STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal dataOut_retrasado2_sig :  STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal dataOut_aux_sig :  STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal dataOut_aux_final_sig :  STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal dataOut_aux_final_retrasado1_sig :  STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal dataOut_aux_final_retrasado2_sig :  STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal validOut_sig :  STD_LOGIC := '0';
        signal validOutAux_sig :  STD_LOGIC := '0';
        signal clkOut_sig :  STD_LOGIC := '0';
        signal data_gen_sig :  STD_LOGIC := '0';
        signal eTx_sig:  STD_LOGIC := '0';
        signal pTx_sig :  STD_LOGIC := '0';
        signal rstGen_sig :  STD_LOGIC := '0';
        --Seales para adecuacion de ciclos de reloj
        signal numero_ciclos_retraso_preamble : STD_LOGIC_VECTOR (2 downto 0) := "000"; --el numero de bit coincide con el de ciclos de reloj
        -------------------------------------------
        --Preamble Generator
        -------------------------------------------
        --Inputs
        --Outputs
        signal preamble_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal preamble_ended_sig :  STD_LOGIC := '0';
        signal preamble_valid_sig :  STD_LOGIC := '0';
        signal preamble_retrasado1_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal preamble_retrasado2_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal preamble_retrasado3_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal preamble_ended_retrasado1_sig :  STD_LOGIC := '0';
        signal preamble_ended_retrasado2_sig :  STD_LOGIC := '0';
        -------------------------------------------
        --Payload Generator
        -------------------------------------------
        --Inputs
        --Outputs
        signal payload_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal payload_retrasado1_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal payload_retrasado2_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal payload_retrasado3_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal payload_ended_header_sig :  STD_LOGIC := '0';
        signal payload_ended_data_sig :  STD_LOGIC := '0';
        -------------------------------------------
        --FSC Generator
        -------------------------------------------
        --Inputs
        --Outputs
        signal fsc_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal fsc_enable_retrasado1 : STD_LOGIC := '0';
        signal fsc_enable_retrasado2 : STD_LOGIC := '0';
        signal fsc_enable_retrasado3 : STD_LOGIC := '0';
        signal fsc_enable_retrasado4 : STD_LOGIC := '0';
        signal fsc_enable_retrasado5 : STD_LOGIC := '0';
        signal fsc_enable_retrasado6 : STD_LOGIC := '0';
        signal fsc_enable_retrasado7 : STD_LOGIC := '0';
        signal mData_invertido : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal mData_invertido_retrasado1 : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal mData_invertido_retrasado2 : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal mData_invertido_retrasado3 : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal mData_invertido_retrasado4 : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal mData_no_invertido : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal mData_retrasado1 : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal mData_retrasado2 : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal mData_retrasado3 : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal mData_retrasado4 : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal fsc_retrasado_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal fsc_invertido : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal fsc_ended_sig :  STD_LOGIC := '0';
        signal mantener_header_gen : integer range 0 to 3 := 0;
        signal generando_fsc : std_logic := '0';
        signal mantener_data_gen : integer range 0 to 3 := 0;
        signal mantener_header_gen_out : integer range 0 to 3 := 0;
        -------------------------------------------
        --Select Generated Data
        -------------------------------------------
        --Inputs
        --Outputs
        -------------------------------------------
        --Select Generating Block
        -------------------------------------------
        --Inputs
        signal endBlock_sig : STD_LOGIC_VECTOR (4 downto 0) := "00000";
        --Outputs
        signal header_gen_sig :  STD_LOGIC := '0';
        signal selBlock_sig : STD_LOGIC_VECTOR (3 downto 0) := "0000";
        signal selBlockAux_sig : STD_LOGIC_VECTOR (3 downto 0) := "0000";
        signal fin_gen_sig      : STD_LOGIC := '0';
        -------------------------------------------
        --Frame Generator Control
        -------------------------------------------
        --Inputs
        --Outputs
        signal mGen_sig : STD_LOGIC := '0';
        signal pInterrupt_sig : STD_LOGIC := '0';
    

        signal salida_datos_retrasada : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal salida_datos : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
        signal mantener_salida_datos :integer range 0 to 4 := 0;
begin
    --------------------------------------------------------------------------------------
    -- Instanciacion de componentes (definicion de puertos y asignacion de seales)
    --------------------------------------------------------------------------------------

        PREAMBLE_GEN: preamble_generator PORT MAP (
                  clk => clk_sig,
                  enable_generate => selBlock_sig(3),
                  --enable_generate => selBlockAux_sig(3),
                  pInterrupt => pInterrupt_sig,
                  eGen => eGen_sig,
                  pGen => pGen_sig,
                  preamble => preamble_sig,
                  ended => preamble_ended_sig,
                  valid => preamble_valid_sig
                );
          
        PAYLOAD_GEN: payload_data_generator PORT MAP (
                clk => clk_sig,
                enable_generate => selBlock_sig(2),
                mData => mData_sig,
                mLaST => mLaST_sig,
                payload => payload_sig,
                ended_header => payload_ended_header_sig,
                ended_data => payload_ended_data_sig
                );  
    
        FSC_GEN: fsc_generator PORT MAP (
              clk => clk_sig,
              --enable => selBlock_sig(1),
              enable => fsc_enable_retrasado6,--6
              --mData => mData_sig,
              --mData => mData_invertido,
              mData => mData_invertido_retrasado4,--4
              --mData => mData_retrasado4,
              mLast => mLast_sig,
              --end_data_gen => payload_ended_data_sig,
              data_gen => data_gen_sig,
              pInterrupt => pInterrupt_sig,
              fsc => fsc_sig,
              ended => fsc_ended_sig
              );
        
        SEL_DATA: select_generated_data PORT MAP (
                  clk => clk_sig,
                  preamble => preamble_retrasado3_sig,
                  --preamble => preamble_sig,
                  payload => payload_sig,
                  --fsc => fsc_sig,
                  fsc => fsc_invertido,
                  --fsc => fsc_retrasado_sig,
                  sel => selBlock_sig,
                  data => dataOut_sig
                  );
                  
         SEL_BLOCK: select_generating_block PORT MAP (
                clk => clk_sig,
                rst => rst_sig,
                mGen => mGen_sig,
                endBlock => endBlock_sig,
                pInterrupt => pInterrupt_sig,
                header_gen => header_gen_sig,
                data_gen => data_gen_sig,
                selBlock => selBlock_sig,
                fin_gen => fin_gen_sig,
                valid => validOut_sig
                );
                
        FRAME_GEN_CONTROL: frame_generator_control PORT MAP (
                  enable => enable_sig,
                  eGen => eGen_sig,
                  pGen => pGen_sig,
                  mValid => mValid_sig,
                  mLast => mLast_sig,
                  rst => rst_sig,
                  clk => clk_sig,
                  fin_gen => fin_gen_sig,
                  mGen => mGen_sig,
                  eTx => eTx_sig,
                  pTx => pTx_sig,
                  rstGen => rstGen_sig,
                  pInterrupt => pInterrupt_sig,
                  ready => readyIn_sig
                  );
    -------------------------------------------
    -- Decodificacion de puertos de salida
    -------------------------------------------
    endBlock_sig <= preamble_ended_sig&payload_ended_header_sig&payload_ended_data_sig&fsc_ended_sig&'0';
    --endBlock_sig <= preamble_ended_retrasado2_sig&payload_ended_header_sig&payload_ended_data_sig&fsc_ended_sig&'0';
    
    process(clk_sig)
    begin
        if (clk_sig'event and clk_sig = '1') then
            --if(preamble_valid_sig = '1') then
--                --se esta generando preamble valido
                preamble_retrasado1_sig <= preamble_sig;
                preamble_retrasado2_sig <= preamble_retrasado1_sig;
                preamble_retrasado3_sig <= preamble_retrasado2_sig;
                
                payload_retrasado1_sig <= payload_sig;
                payload_retrasado2_sig <= payload_retrasado1_sig;
                payload_retrasado3_sig <= payload_retrasado2_sig;
                
                fsc_retrasado_sig <= fsc_sig;
                
                fsc_enable_retrasado1 <= selBlock_sig(1);
                fsc_enable_retrasado2 <= fsc_enable_retrasado1;
                fsc_enable_retrasado3 <= fsc_enable_retrasado2;
                fsc_enable_retrasado4 <= fsc_enable_retrasado3;
                fsc_enable_retrasado5 <= fsc_enable_retrasado4;
                fsc_enable_retrasado6 <= fsc_enable_retrasado5;
                fsc_enable_retrasado7 <= fsc_enable_retrasado6;
                
                if (header_gen_sig = '1') then
                    mData_invertido_retrasado1 <= mData_invertido;
                    mData_invertido_retrasado2 <= mData_invertido_retrasado1;
                    mData_invertido_retrasado3 <= mData_invertido_retrasado2;
                    mData_invertido_retrasado4 <= mData_invertido_retrasado3;
                    mantener_header_gen <= 0;
                else
                    if(mantener_header_gen < 3) then
                        mantener_header_gen <= mantener_header_gen + 1;
                        mData_invertido_retrasado1 <= mData_invertido;
                        mData_invertido_retrasado2 <= mData_invertido_retrasado1;
                        mData_invertido_retrasado3 <= mData_invertido_retrasado2;
                        mData_invertido_retrasado4 <= mData_invertido_retrasado3;
                    else                
                        mData_invertido_retrasado4 <= mData_invertido;
                    end if;
                end if;
                
                if (header_gen_sig = '1') then
                    mData_retrasado1 <= mData_no_invertido;
                    mData_retrasado2 <= mData_retrasado1;
                    mData_retrasado3 <= mData_retrasado2;
                    mData_retrasado4 <= mData_retrasado3;
                else
                    if(mantener_header_gen < 3) then
                        mData_retrasado1 <= mData_no_invertido;
                        mData_retrasado2 <= mData_retrasado1;
                        mData_retrasado3 <= mData_retrasado2;
                        mData_retrasado4 <= mData_retrasado3;
                    else                
                        mData_retrasado4 <= mData_no_invertido;
                    end if;
                end if;
                
                
                --gestion de ensamblado de trama para enganche con fsc
                if (pInterrupt_sig = '0')then
                    if (header_gen_sig = '1' and data_gen_sig = '0') then
                        -- generando payload header (10)
                        dataOut_retrasado1_sig <= dataOut_sig;
                        dataOut_retrasado2_sig <= dataOut_retrasado1_sig;
                        dataOut_aux_sig <= dataOut_retrasado2_sig;
                        generando_fsc <= '0';
                    else
                        if (header_gen_sig = '0' and data_gen_sig = '1') then
                            -- (01)
                            if (validOut_sig = '0') then
                                generando_fsc <= '0';
                            end if;
                                                    
                            if (mantener_header_gen = 0) then
                                --mantenimiento de seal para no perder octetos
                                dataOut_retrasado1_sig <= dataOut_sig;
                                dataOut_retrasado2_sig <= dataOut_retrasado1_sig;
                                dataOut_aux_sig <= dataOut_retrasado2_sig;
                            else
                                if (payload_ended_data_sig = '1') then
                                    -- flag activacion generacion fsc
                                    generando_fsc <= '1';
                                end if;
                                
                                --mantenimiento de seal para no perder octetos
                                if (mantener_header_gen = 1) then
                                    dataOut_retrasado1_sig <= dataOut_sig;
                                    dataOut_aux_sig <= dataOut_retrasado2_sig;
                                else
                                    if (generando_fsc = '1') then
                                        -- generando fsc
                                        dataOut_aux_sig(0) <= fsc_sig(7);
                                        dataOut_aux_sig(1) <= fsc_sig(6);
                                        dataOut_aux_sig(2) <= fsc_sig(5);
                                        dataOut_aux_sig(3) <= fsc_sig(4);
                                        dataOut_aux_sig(4) <= fsc_sig(3);
                                        dataOut_aux_sig(5) <= fsc_sig(2);
                                        dataOut_aux_sig(6) <= fsc_sig(1);
                                        dataOut_aux_sig(7) <= fsc_sig(0);
                                    else
                                        --generando payload data, preamble o cabecera payload
                                        dataOut_retrasado1_sig <= dataOut_sig;
                                        dataOut_retrasado2_sig <= dataOut_retrasado1_sig;
                                        dataOut_aux_sig <= dataOut_retrasado2_sig;
                                    end if;
                                end if;
                            end if;
                        else
                            -- (00)
                            if (validOut_sig = '0') then
                                generando_fsc <= '0';
                            end if;
                            
                            if (generando_fsc = '1') then
                                -- generando fsc
                                dataOut_aux_sig(0) <= fsc_sig(7);
                                dataOut_aux_sig(1) <= fsc_sig(6);
                                dataOut_aux_sig(2) <= fsc_sig(5);
                                dataOut_aux_sig(3) <= fsc_sig(4);
                                dataOut_aux_sig(4) <= fsc_sig(3);
                                dataOut_aux_sig(5) <= fsc_sig(2);
                                dataOut_aux_sig(6) <= fsc_sig(1);
                                dataOut_aux_sig(7) <= fsc_sig(0);
                            else
                                --generando payload data, preamble o cabecera payload
                                dataOut_retrasado1_sig <= dataOut_sig;
                                dataOut_retrasado2_sig <= dataOut_retrasado1_sig;
                                dataOut_aux_sig <= dataOut_retrasado2_sig;
                            end if;
                        end if;
                    end if;
                else
                --if (pInterrupt_sig = '1') --> solo tenemos en cuenta el paso de payload a fsc
                        if (header_gen_sig = '0' and data_gen_sig = '1') then
                            -- (01)
                            if (validOut_sig = '0') then
                                generando_fsc <= '0';
                            end if;
                                                    
                            if (mantener_header_gen = 0) then
                                --mantenimiento de seal para no perder octetos
                                dataOut_retrasado1_sig <= dataOut_sig;
                                dataOut_retrasado2_sig <= dataOut_retrasado1_sig;
                                dataOut_aux_sig <= dataOut_retrasado2_sig;
                            else
                                if (payload_ended_data_sig = '1') then
                                    -- flag activacion generacion fsc
                                    generando_fsc <= '1';
                                end if;
                                
                                --mantenimiento de seal para no perder octetos
                                if (mantener_header_gen = 1) then
                                    dataOut_retrasado1_sig <= dataOut_sig;
                                    dataOut_aux_sig <= dataOut_retrasado2_sig;
                                else
                                    if (generando_fsc = '1') then
                                        -- generando fsc
                                        dataOut_aux_sig(0) <= fsc_sig(7);
                                        dataOut_aux_sig(1) <= fsc_sig(6);
                                        dataOut_aux_sig(2) <= fsc_sig(5);
                                        dataOut_aux_sig(3) <= fsc_sig(4);
                                        dataOut_aux_sig(4) <= fsc_sig(3);
                                        dataOut_aux_sig(5) <= fsc_sig(2);
                                        dataOut_aux_sig(6) <= fsc_sig(1);
                                        dataOut_aux_sig(7) <= fsc_sig(0);
                                    else
                                        --generando payload data, preamble o cabecera payload
                                        dataOut_retrasado1_sig <= dataOut_sig;
                                        dataOut_retrasado2_sig <= dataOut_retrasado1_sig;
                                        dataOut_aux_sig <= dataOut_retrasado2_sig;
                                    end if;
                                end if;
                            end if;
                        else
                            -- (00)
                            if (validOut_sig = '0') then
                                generando_fsc <= '0';
                            end if;
                            
                            if (pInterrupt_sig = '0') then
                                if (generando_fsc = '1') then
                                    -- generando fsc
                                    dataOut_aux_sig(0) <= fsc_sig(7);
                                    dataOut_aux_sig(1) <= fsc_sig(6);
                                    dataOut_aux_sig(2) <= fsc_sig(5);
                                    dataOut_aux_sig(3) <= fsc_sig(4);
                                    dataOut_aux_sig(4) <= fsc_sig(3);
                                    dataOut_aux_sig(5) <= fsc_sig(2);
                                    dataOut_aux_sig(6) <= fsc_sig(1);
                                    dataOut_aux_sig(7) <= fsc_sig(0);
                                else
                                    --generando payload data final
                                    dataOut_aux_sig <= dataOut_sig;
                                end if;
                            else
                                dataOut_retrasado1_sig <= dataOut_sig;
                                dataOut_retrasado2_sig <= dataOut_retrasado1_sig;
                                dataOut_aux_sig <= dataOut_retrasado2_sig;
                            end if;
                        end if;
                
                end if;
                
                
                --validOutAux_sig <= validOut_sig;
--                if (preamble_ended_sig = '1' and selBlock_sig(2) = '1') then
--                    selBlockAux_sig <= selBlockAux_sig;
--                else
--                    selBlockAux_sig <= selBlock_sig;
--                end if;
                
                
--                preamble_ended_retrasado1_sig <= preamble_ended_sig;
--                preamble_ended_retrasado2_sig <= preamble_ended_retrasado1_sig;
                
                
----                if(numero_ciclos_retraso_preamble = "111") then
----                    preamble_retrasado_sig <= preamble_sig;
----                else
----                    numero_ciclos_retraso_preamble <= '1'&numero_ciclos_retraso_preamble( 2 downto 1 );
----                    preamble_retrasado_sig <= "00000000";
----                end if;
----            else
----                --no se esta generando preamble valido por lo que volvemos al estado inicial
----                if(numero_ciclos_retraso_preamble = "000") then
----                    preamble_retrasado_sig <= "00000000";
----                else
----                    numero_ciclos_retraso_preamble <= '0'&numero_ciclos_retraso_preamble( 2 downto 1 );
----                    preamble_retrasado_sig <= preamble_sig;
----                end if;
--            --end if;
        end if;
    end process;
    
    process(clk_sig)
        begin
            if(clk_sig'event and clk_sig = '1') then
                if (payload_ended_header_sig = '1') then
                    mData_invertido(0) <= mData_sig(7);
                    mData_invertido(1) <= mData_sig(6);
                    mData_invertido(2) <= mData_sig(5);
                    mData_invertido(3) <= mData_sig(4);
                    mData_invertido(4) <= mData_sig(3);
                    mData_invertido(5) <= mData_sig(2);
                    mData_invertido(6) <= mData_sig(1);
                    mData_invertido(7) <= mData_sig(0);
                    mData_no_invertido <= mData_sig;
                else
                    mData_invertido(0) <= payload_sig(7);
                    mData_invertido(1) <= payload_sig(6);
                    mData_invertido(2) <= payload_sig(5);
                    mData_invertido(3) <= payload_sig(4);
                    mData_invertido(4) <= payload_sig(3);
                    mData_invertido(5) <= payload_sig(2);
                    mData_invertido(6) <= payload_sig(1);
                    mData_invertido(7) <= payload_sig(0);
                    mData_no_invertido <= payload_sig;
                end if;
                
                fsc_invertido(0) <= fsc_sig(7);
                fsc_invertido(1) <= fsc_sig(6);
                fsc_invertido(2) <= fsc_sig(5);
                fsc_invertido(3) <= fsc_sig(4);
                fsc_invertido(4) <= fsc_sig(3);
                fsc_invertido(5) <= fsc_sig(2);
                fsc_invertido(6) <= fsc_sig(1);
                fsc_invertido(7) <= fsc_sig(0);
                
                
                if (validOut_sig = '1') then
                    if (generando_fsc = '0') then
                        if (header_gen_sig = '0') then
                            if (mantener_header_gen_out < 4) then
                                mantener_header_gen_out <= mantener_header_gen_out + 1;
                                dataOut_aux_final_retrasado1_sig <= dataOut_aux_sig;
                                --dataOut_aux_final_retrasado2_sig <= dataOut_aux_final_retrasado1_sig;
                                dataOut_aux_final_sig <= dataOut_aux_final_retrasado1_sig;                      
                            else
                                dataOut_aux_final_sig <= dataOut_sig;
                                
                                if(payload_ended_data_sig = '1') then
                                    mantener_data_gen <= 0;
                                end if; 
                            end if;
                        else
                            dataOut_aux_final_retrasado1_sig <= dataOut_aux_sig;
                            --dataOut_aux_final_retrasado2_sig <= dataOut_aux_final_retrasado1_sig;
                            dataOut_aux_final_sig <= dataOut_aux_final_retrasado1_sig;
                        end if;
                                              
                    else
                        --mantener 2 ciclos y continuar con dataOut                 
                        
                        if (mantener_data_gen < 2) then
                            mantener_data_gen <= mantener_data_gen + 1;
                            dataOut_aux_final_sig <= dataOut_sig;
                        else
                            dataOut_aux_final_sig(0) <= fsc_sig(7);
                            dataOut_aux_final_sig(1) <= fsc_sig(6);
                            dataOut_aux_final_sig(2) <= fsc_sig(5);
                            dataOut_aux_final_sig(3) <= fsc_sig(4);
                            dataOut_aux_final_sig(4) <= fsc_sig(3);
                            dataOut_aux_final_sig(5) <= fsc_sig(2);
                            dataOut_aux_final_sig(6) <= fsc_sig(1);
                            dataOut_aux_final_sig(7) <= fsc_sig(0);
                        end if;
                   
                    end if;
                else
                    dataOut_aux_final_retrasado1_sig <= dataOut_aux_sig;
                    dataOut_aux_final_retrasado2_sig <= dataOut_aux_final_retrasado1_sig;
                    dataOut_aux_final_sig <= dataOut_aux_final_retrasado2_sig;
                    mantener_data_gen <= 4;
                    mantener_header_gen_out <= 0;
                end if;
            end if;
--            if(clk_sig'event and clk_sig = '1') then
--                if (payload_ended_header_sig = '1') then
--                    mData_invertido <= mData_sig;
--                else
--                    mData_invertido <= payload_sig;
--                end if;
--            end if;
            


        end process;
        
        process(clk_sig)
        begin
            if(clk_sig'event and clk_sig = '1') then
                if (validOut_sig = '1') then
                    if (pInterrupt_sig = '0') then
                        if (header_gen_sig = '1' or data_gen_sig = '1') then
                            if (preamble_ended_sig = '1') then
                                --primera parte de la trama
                                salida_datos <= dataOut_aux_final_sig;
                                salida_datos_retrasada <= dataOut_aux_final_sig;
                            else
                                salida_datos_retrasada <= dataOut_aux_final_sig;
                                salida_datos <= salida_datos_retrasada;
                            end if;
                        else
                            if (payload_ended_data_sig = '1') then  
                                salida_datos_retrasada <= dataOut_aux_final_sig;
                                salida_datos <= salida_datos_retrasada;
                            else
                                if (mantener_salida_datos < 1) then
                                    mantener_salida_datos <= mantener_salida_datos + 1;
                                    salida_datos_retrasada <= dataOut_aux_final_sig;
                                    salida_datos <= salida_datos_retrasada;
                                else
                                    salida_datos <= fsc_invertido;
                                    
                                end if;
                            end if;
                        end if;
                    else
                        if (data_gen_sig = '1') then
--                                salida_datos_retrasada <= dataOut_aux_sig;
--                                salida_datos <= salida_datos_retrasada;
                                --salida_datos <= dataOut_aux_sig;
                                salida_datos <= payload_retrasado3_sig; 
                        else
                            if (mantener_salida_datos < 3) then
                                mantener_salida_datos <= mantener_salida_datos + 1;
--                                salida_datos_retrasada <= dataOut_aux_sig;
--                                salida_datos <= salida_datos_retrasada;
                                salida_datos <= payload_retrasado3_sig; 
                            else
                                if (mantener_salida_datos = 3) then
                                    mantener_salida_datos <= mantener_salida_datos + 1;
                                    salida_datos <= payload_retrasado3_sig;
                                else
                                
--                                salida_datos(0) <= fsc_sig(7);
--                                salida_datos(1) <= fsc_sig(6);
--                                salida_datos(2) <= fsc_sig(5);
--                                salida_datos(3) <= fsc_sig(4);
--                                salida_datos(4) <= fsc_sig(3);
--                                salida_datos(5) <= fsc_sig(2);
--                                salida_datos(6) <= fsc_sig(1);
--                                salida_datos(7) <= fsc_sig(0);
                                    salida_datos <= fsc_invertido;
                                end if;
                            end if;
                        end if;
                    end if;
                else
                    -- si validOut = 0
                    salida_datos <= "00000000";
                    mantener_salida_datos <= 0;
                end if;  
            end if;
        end process;
    --------------------------------------------------------------------------------------
    -- Asignaciones de seales a puertos y de puertos a seales
    --------------------------------------------------------------------------------------
    -- Inputs
    mLength_sig <= mLengthFG;
    mData_sig <= mDataFG;
    mValid_sig <= mValidFG;
    mLast_sig <= mLastFG;
    eGen_sig <= eGenFG;
    pGen_sig <= pGenFG;
    mLengthTotal_sig <= mLengthTotalFG;
    enable_sig <= enableFG;
    rst_sig <= rstFG;
    clk_sig <= clkFG;
    readyIn_sig <= readyInFG; 
             
    --Outputs
    --dataOutFG <= dataOut_sig;
    --dataOutFG <= dataOut_aux_sig;
    --dataOutFG <= dataOut_aux_final_sig;
    dataOutFG <= salida_datos;
    validOutFG <= validOut_sig;
    --validOutFG <= validOutAux_sig;
    clkOutFG <= clk_sig;
    header_genFG <= header_gen_sig;
    data_genFG <= data_gen_sig;
    eTxOutFG <= eTx_sig;
    pTxOutFG <= pTx_sig;
    rstGenFG <= rstGen_sig;
   

end struct;
