----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 01.03.2019 20:02:28
-- Design Name: 
-- Module Name: contador_dsc - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity contador_dsc is
    Port ( enable : in STD_LOGIC;
           rst : in STD_LOGIC;
           start_count_val : in STD_LOGIC_VECTOR (7 downto 0);
           generating : out STD_LOGIC;
           ending : out STD_LOGIC;
           mLength : out STD_LOGIC_VECTOR (7 downto 0);
           clk : in STD_LOGIC);
end contador_dsc;

architecture Behavioral of contador_dsc is

    -- seales internas para las entradas del bloque
    signal clk_sig : std_logic := '0';
    signal rst_sig : std_logic := '0';
    signal enable_sig : std_logic := '0';
    signal start_count_val_sig : std_logic_vector (7 downto 0) := "00000000";
    

    -- seales internas para las salidas del bloque
    signal generating_sig: std_logic := '0';
    signal ending_sig : std_logic := '0';
    signal packet_length_sig: std_logic_vector (7 downto 0) := "00000000";
    
    
    
    -- seales internas para la funcion del contador
    signal count_val: integer range 0 to 255 := 0;               -- Valor del contador (en octetos)
    constant count_desc_val: integer range 0 to 255 := 1;        -- Valor de decremento del contador (tamao de generacion -> 1octeto/ciclo)

begin

process (clk_sig)
begin
    if (clk_sig'event and clk_sig='1') then
    -- si tiene lugar evento de reloj y en pulso ascendente
        if (rst_sig = '1') then
        -- en caso de reset (inicializacion)
                count_val <= to_integer(unsigned(start_count_val));
                generating_sig <= '0';
                ending_sig <= '0';
                packet_length_sig <= "00000000";
        else
        -- en caso de funcionamiento normal
            if (enable_sig = '1') then
            -- habilitacion activada
                if (count_val = 0) then
                -- fin de conteo
                    generating_sig <= '0';
                    ending_sig <= '0';
                    packet_length_sig <= "00000000";
                else  
                -- en conteo
                    if (count_val <= 1) then   
                    -- ultima cuenta (enviamos longitud de ultimo paquete e indicamos fin)            
                        packet_length_sig <= std_logic_vector(to_unsigned(count_val,mLength'length));   -- convertimos a std_logic_vector con valor sin signo del entero (tamao restante de paquete) y con la longitud del vector destino
                        count_val <= 0;
                        ending_sig <= '1';
                    else
                    -- cuenta normal (enviamos longitud de paquete e indicamos generacion sin fin)
                        packet_length_sig <= std_logic_vector(to_unsigned(count_desc_val,mLength'length));  -- convertimos a std_logic_vector con valor sin signo del entero (valor maximo de paquete) y con la longitud del vector destino
                        count_val <= (count_val - count_desc_val);
                        ending_sig <= '0';
                    end if;
                    -- en ambos casos indicamos generacion de paquete
                    generating_sig <= '1';
                end if;
            end if;
        end if;
    end if;
end process;


-- asignacion de valores de seales internas desde los puertos de entrada
clk_sig <= clk;
rst_sig <= rst;
enable_sig <= enable;
start_count_val_sig <= start_count_val;

-- asignacion de valores a los puertos mediante las seales internas
generating <= generating_sig; 
ending <= ending_sig;
mLength <= packet_length_sig;

end Behavioral;
