----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 09.03.2019 13:12:13
-- Design Name: 
-- Module Name: control_tipo_paquete - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity control_tipo_paquete is
    Port ( rst : in STD_LOGIC;
           clk : in STD_LOGIC;
           enable : in STD_LOGIC;
           mSel : out STD_LOGIC_VECTOR (1 downto 0);
           mLength : out STD_LOGIC_VECTOR (7 downto 0));
end control_tipo_paquete;

architecture Behavioral of control_tipo_paquete is

    -- seales internas para las entradas del bloque
    signal rst_sig: std_logic := '0';
    signal clk_sig: std_logic := '0';
    signal enable_sig: std_logic := '0';

    -- seales internas para las salidas del bloque
    signal mSel_sig: std_logic_vector (1 downto 0) := "00";
    signal mLength_sig: std_logic_vector (7 downto 0):= "00000000";
    
    -- seales internas para generacion de paquetes
    constant no_type : std_logic_vector (1 downto 0) :=  "00";
    constant p_type  : std_logic_vector (1 downto 0) :=  "01";
    constant e_type_1: std_logic_vector (1 downto 0) :=  "10";
    constant e_type_2: std_logic_vector (1 downto 0) :=  "11";
    
    signal contador: integer range 0 to 1500 := 0;       -- Numero de ciclos para definicion del tipo de paquete a generar en cada momento (no aleatorio)
    --signal contador_freq: integer range 0 to 25000;   -- Numero de cuentas para un periodo (20000ps -> 50MHz)  
    --constant contador_freq_max: integer:= 20000;      -- Valor maximo de conteos para cada periodo
    
    constant intervalo_1 : integer:= 10;                -- Numero de ciclos para cada uno de los intervalos a definir
    constant intervalo_2 : integer:= 310;                
    constant intervalo_3 : integer:= 610;
    constant intervalo_4 : integer:= 710;
    constant intervalo_5 : integer:= 910;
    constant intervalo_6 : integer:= 1110;
    constant gap_flag_sel : integer := 50;
    
    -- Tamaos de paquetes para pruebas 
    constant no_packet : std_logic_vector (7 downto 0)     :=  "00000000";     -- 0 oct  
    constant small_packet : std_logic_vector (7 downto 0)  :=  "01111000";     -- 120 oct       
    constant medium_packet : std_logic_vector (7 downto 0) :=  "11111111";     -- 254 oct
    constant big_packet : std_logic_vector (7 downto 0)    :=  "01111000";     -- 120 oct
    
begin

process(clk_sig)
begin
    if (clk_sig'event and clk_sig='1') then
    -- si tiene lugar evento de reloj y en pulso ascendente
        if (rst_sig = '1') then
        -- en caso de reset (inicializacion)
              mSel_sig <= no_type; 
              mLength_sig <= no_packet;
              contador <= 0;
        else
        -- en caso de funcionamiento normal 
            if (enable_sig = '1') then
            -- habilitacion activada
                --contador_freq <= contador_freq +1;
                
                --if(contador_freq = contador_freq_max) then
                    contador <= contador + 1;
                    --contador_freq <= 0;
                --end if;
                     case contador is
                         when intervalo_1 => 
                            --mLength_sig <= medium_packet;
                            mLength_sig <= small_packet;
                            mSel_sig <= e_type_1;                        
                         when (intervalo_1 + gap_flag_sel) => 
                            --mLength_sig <= medium_packet;
                            mSel_sig <= no_type;
                         when intervalo_2 =>
                            --mLength_sig <= medium_packet;
                            mLength_sig <= small_packet;
                            mSel_sig <= p_type;
                         when (intervalo_2 + gap_flag_sel) => 
                            --mLength_sig <= medium_packet;
                            mSel_sig <= no_type;
                         when intervalo_3 =>
                            mLength_sig <= medium_packet;
                            mSel_sig <= p_type;
                         when (intervalo_3 + gap_flag_sel) => 
                            --mLength_sig <= medium_packet;
                            mSel_sig <= no_type;
                         when intervalo_4 =>
                            mLength_sig <= small_packet;
                            mSel_sig <= e_type_2;
                         when (intervalo_4 + gap_flag_sel) => 
                            --mLength_sig <= medium_packet;
                            mSel_sig <= no_type;
                         when intervalo_5 =>
                            mLength_sig <= small_packet;
                            mSel_sig <= p_type;
                         when (intervalo_5 + gap_flag_sel) => 
                            --mLength_sig <= medium_packet;
                            mSel_sig <= no_type;
                         when intervalo_6 =>
                            mLength_sig <= no_packet;
                            mSel_sig <= no_type;
                         when others =>
                            mLength_sig <= mLength_sig;
                            mSel_sig <= mSel_sig;
                     end case;
            else
                
            end if;
        end if;
    end if;
end process;

-- asignacion de valores de seales internas mediante los puertos
rst_sig <= rst;
clk_sig <= clk;
enable_sig <= enable;

-- asignacion de valores a los puertos mediante las seales internas
mSel <= mSel_sig; 
mLength <= mLength_sig;

end Behavioral;
