----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 03.04.2019 15:20:37
-- Design Name: 
-- Module Name: frame_generator_control - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity frame_generator_control is
    Port ( enable : in STD_LOGIC;
           eGen : in STD_LOGIC;
           pGen : in STD_LOGIC;
           mValid : in STD_LOGIC;
           mLast : in STD_LOGIC;
           rst : in STD_LOGIC;
           clk : in STD_LOGIC;
           fin_gen : in STD_LOGIC;
           mGen : out STD_LOGIC;
           eTx : out STD_LOGIC;
           pTx : out STD_LOGIC;
           rstGen : out STD_LOGIC;
           pInterrupt: out STD_LOGIC;
           ready : in STD_LOGIC);
end frame_generator_control;

architecture Behavioral of frame_generator_control is

    type estados is (espera, init_Express, init_Preemptable, gen_Express, gen_Preemptable, finalizando_Express, finalizando_Preemptable, finalizando_Preemptable_para_Express, fin_Express, fin_Preemptable, fin_Preemptable_para_Express);
        
    -- seales para estados
    signal ea : estados;    -- seal para estado actual
    signal es : estados;    -- seal para estado siguiente
    
    -- seales internas para entradas
    signal enable_sig : STD_LOGIC := '0';
    signal eGen_sig : STD_LOGIC := '0';
    signal pGen_sig : STD_LOGIC := '0';
    signal mValid_sig : STD_LOGIC := '0';
    signal mLast_sig : STD_LOGIC := '0';
    signal rst_sig : STD_LOGIC := '0';
    signal clk_sig : STD_LOGIC := '0';
    signal ready_sig : STD_LOGIC := '0';
    signal fin_gen_sig : STD_LOGIC := '0';
    
    --seales internas para salidas
    signal mGen_sig : STD_LOGIC := '0';
    signal eTx_sig : STD_LOGIC := '0';
    signal pTx_sig : STD_LOGIC := '0';
    signal rstGen_sig : STD_LOGIC := '0';
    signal pInterrupt_sig : STD_LOGIC := '0';
    --signal valid_sig : STD_LOGIC := '0';
    
                                                 
begin

------------------------------------------------------------
------------	MAQUINA ESTADOS: FLIP-FLOPS	---------------
------------------------------------------------------------
    process (clk_sig)
    begin      
         if (clk_sig'event and clk_sig='1') then
            if (rst_sig = '1') then
                -- inicializacion a estado inicial
                 ea <= espera;
             else
            -- funcionamiento normal, pasamos a siguiente estado
                ea <= es;
            end if;        
        end if;
    end process;
    
------------------------------------------------------------
----------	MAQUINA ESTADOS: CONTROL ENTRADAS	------------
------------------------------------------------------------
    process (clk_sig, ea)
    begin
        
        if (clk_sig'event and clk_sig='1') then
        -- seleccion del estado siguiente en funcion del estado actual y de las entradas en dicho momento
            if (enable_sig = '1') then
            -- el bloque esta habilitado
                case ea is     
                    when espera =>
                    -- estado inicial (1)
                        if (eGen_sig = '1') then
                        -- seleccion de generacion de Express Packet
                            es <= init_Express;
                        else
                            if (pGen_sig = '1') then
                            -- seleccion de generacion de Preemtable Packet
                                es <= init_Preemptable;
                            else
                                es <= ea;
                            end if;
                        end if;
                    
                    when init_Express =>
                    -- estado de inicializacion para la generacion de paquete Express (2)
                        es <= gen_Express;
                    
                    when init_Preemptable =>
                    -- estado de inicializacion para la generacion de paquete Preemtable (3)
                       es <= gen_Preemptable;
                        
                    when gen_Express =>
                    -- estado de generacion de paquete Express (4)
                        if (fin_gen_sig = '0') then
                            if (mValid_sig = '1') then
                            -- generacion de subpaquete correcta
                                if (mLast_sig = '1') then
                                -- llega ultimo subpaquete de paquete Express
                                    es <= finalizando_Express;
                                else 
                                -- seguimos generando Express Packet
                                    es <= gen_Express;
                                end if;
                            else
                            -- error en generacion de subpaquete
                                --es <= ???????????????????????????;
                            end if;
                        else
                            --finalizacion indicada por bloque de ensamblado
                            es <= finalizando_Express;
                        end if;
                        
                    when gen_Preemptable =>
                    -- estado de generacion de paquete Preemtable (5)
                        if (eGen_sig = '1') then
                        -- llega paquete Express y se detiene trasnmision de paquete Preemptable
                            es <= finalizando_Preemptable_para_Express; 
                        else
                        -- generacion normal de paquete Preemtable
                            if (fin_gen_sig = '0') then
                                if (mValid_sig = '1') then
                                -- generacion de subpaquete correcta
                                    if (mLast_sig = '1') then
                                    -- llega ultimo subpaquete de paquete Preemtable
                                        es <= finalizando_Preemptable;
                                    else 
                                    -- seguimos generando Preemptable Packet
                                        es <= gen_Preemptable;
                                    end if;
                                else
                                -- error en generacion de subpaquete
                                    --es <= ???????????????????????????;
                                end if; 
                            else
                                --finalizacion indicada por bloque de ensamblado
                                es <= finalizando_Express;
                            end if;                      
                        end if;
                        
                    when finalizando_Express =>
                    -- estado de espera a finalizacion de la generacion de paquete Express (6)
                        if (fin_gen_sig = '1') then
                            --esperamos a que bloque selector de bloques indique finalizacion de ensamblado de trama
                            es <= fin_Express;
                        else
                            es <= ea;
                        end if;
                        
                    when finalizando_Preemptable =>
                    -- estado de espera a finalizacion de la generacion de paquete Preemtable (7)
                        if (fin_gen_sig = '1') then
                            --esperamos a que bloque selector de bloques indique finalizacion de ensamblado de trama
                            es <= fin_Preemptable;
                        else
                            es <= ea;
                        end if;
                    
                    when finalizando_Preemptable_para_Express =>
                    -- estado de espera a finalizacion de la generacion de paquete Preemtable por la llegada de orden de generacion de paquete Express (8)
                        --es <= espera;
                        if (fin_gen_sig = '1') then
                            --esperamos a que bloque selector de bloques indique finalizacion de ensamblado de trama
                            es <= fin_Preemptable_para_Express;
                        else
                            es <= ea;
                        end if;    
                        
                    when fin_Express =>
                    -- estado de finalizacion de la generacion de paquete Express (9)
                        es <= espera;
                        
                    when fin_Preemptable =>
                    -- estado de finalizacion de la generacion de paquete Preemtable (10)
                        es <= espera;
                    
                    when fin_Preemptable_para_Express =>
                    -- estado de finalizacion de la generacion de paquete Preemtable por la llegada de orden de generacion de paquete Express (11)
                        es <= init_Express;
                    
                end case;
            end if; 
        end if;       
    end process;
	

------------------------------------------------------------
----------	MAQUINA ESTADOS: CONTROL SALIDAS	------------
------------------------------------------------------------
	WITH ea SELECT
        mGen_sig <=         '0' when espera,
                            '0' when init_Express,
                            '0' when init_Preemptable,
                            '0' when fin_Express,
                            '0' when fin_Preemptable,
                            '0' when fin_Preemptable_para_Express,
                            '1' when others;
	
	WITH ea SELECT
        eTx_sig <=          '1' when fin_Express,
                            '0' when others;
		
						
	WITH ea SELECT
        pTx_sig <=          '1' when fin_Preemptable,
                            '1' when fin_Preemptable_para_Express,
                            '0' when others;
                    
    WITH ea SELECT
        rstGen_sig <=       '1' when init_Express,
                            '1' when init_Preemptable,
                            '0' when others;
                        
    WITH ea SELECT
        pInterrupt_sig <=   '1' when finalizando_Preemptable_para_Express,
                            '0' when others;
                            
--    WITH ea SELECT
--        valid_sig <=        '1' when gen_Express,
--                            '1' when gen_Preemptable,
--                            '1' when finalizando_Express,
--                            '1' when finalizando_Preemptable,
--                            '1' when finalizando_Preemptable_para_Express,
--                            '0' when others;
                            
					
						
--------------------------------------------------------------------------------
--- asignacion de valores de seales internas mediante los puertos    ----------
--------------------------------------------------------------------------------		
enable_sig <= enable;
eGen_sig <= eGen;
pGen_sig <= pGen;
mValid_sig <= mValid;
mLast_sig <= mLast;
rst_sig <= rst;
clk_sig <= clk;
ready_sig <= ready;
fin_gen_sig <= fin_gen;

--------------------------------------------------------------------------------
--- asignacion de valores de los puertos mediante las seales internas    ------
--------------------------------------------------------------------------------
mGen <= mGen_sig;
eTx <= eTx_sig;
pTx <= pTx_sig;
rstGen <= rstGen_sig;
pInterrupt <= pInterrupt_sig;
--valid <= valid_sig;
    

end Behavioral;
