----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 08.06.2019 19:31:03
-- Design Name: 
-- Module Name: fsc_generator - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

--library xil_defaultlib;
--use xil_defaultlib.PCK_CRC32_D8.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity fsc_generator is
    Port ( clk : in STD_LOGIC;
           enable : in STD_LOGIC;
           mData : in STD_LOGIC_VECTOR (7 downto 0);
           mLast : in STD_LOGIC;
           --end_data_gen : in STD_LOGIC;
           data_gen : in STD_LOGIC;
           pInterrupt : in STD_LOGIC;
           fsc : out STD_LOGIC_VECTOR (7 downto 0);
           ended : out STD_LOGIC);
end fsc_generator;

architecture Behavioral of fsc_generator is
    --seales internas para entradas
    signal clk_sig :  STD_LOGIC := '0';
    signal enable_sig :  STD_LOGIC := '0';
    signal mData_sig :  STD_LOGIC_VECTOR(7 downto 0) := "00000000";
    signal mLast_sig : STD_LOGIC:= '0';
    signal pInterrupt_sig : STD_LOGIC:= '0';
    --signal end_data_gen_sig : STD_LOGIC := '0';
    signal data_gen_sig : STD_LOGIC := '0';
    signal end_data_gen_sig_ciclo_anterior : STD_LOGIC := '0';
    --seales internas para salidas
    signal fsc_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
    signal fsc_aux_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
    signal ended_sig :  STD_LOGIC := '0';
    
    --seales internas para calculo de CRC
    signal crc : STD_LOGIC_VECTOR (31 downto 0) := "11111111111111111111111111111111";
    
    --seales internar para generacion de salida de CRC
    signal emitiendo_crc_condicion_1 : STD_LOGIC := '0';    --mlast 
    signal emitiendo_crc_condicion_2 : STD_LOGIC := '0';    --end_data_gen
    signal octetos_emitidos: integer range 0 to 5 := 0;
    
    --constantes para calculo de mCRC de fragmentos interrumpidos
    constant octetos_1_2_XOR : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
    constant octetos_3_4_XOR : STD_LOGIC_VECTOR (7 downto 0) := "11111111";
    
    
    ----------------------------------------------------------------------------------
    signal      Init       :   std_logic := '1';       --! Initialize CRC generation 
    signal      Frame_data :   std_logic_vector(7 downto 0) := "00000000";  --! Input data
    signal      Data_en    :   std_logic := '0';       --! Input data valid
    signal      CRC_rd     :   std_logic := '0';       --! Read calculated CRC
    signal      CRC_out    :  std_logic_vector(7 downto 0) := (others => '0');
    --------------------------------------------------------------------------------
    
      -----------------------------------------------------------------------------
      -- REGISTERS
      -----------------------------------------------------------------------------
      signal CRC_reg : std_logic_vector(31 downto 0) := (others => '1');
      signal CRC_reg_not : std_logic_vector(31 downto 0) := (others => '1');
    
      -----------------------------------------------------------------------------
      -- FUNCTION
      -----------------------------------------------------------------------------
      function parallel_crc (crc_in  : std_logic_vector(31 downto 0);
                             data_in : std_logic_vector(7 downto 0)
                             ) return std_logic_vector is
        variable C, NewCRC : std_logic_vector(31 downto 0) := (others => '1');
        variable D         : std_logic_vector (7 downto 0) := (others => '0');
    
      begin
    
        C := crc_in;
        D := data_in;
    
    
    -- The parellel CRC is a function of the X vector and the current CRC.
        NewCRC(0)  := C(24) xor C(30) xor D(1) xor D(7);
        NewCRC(1)  := C(25) xor C(31) xor D(0) xor D(6) xor C(24) xor C(30) xor D(1) xor D(7);
        NewCRC(2)  := C(26) xor D(5) xor C(25) xor C(31) xor D(0) xor D(6) xor C(24) xor C(30) xor D(1) xor D(7);
        NewCRC(3)  := C(27) xor D(4) xor C(26) xor D(5) xor C(25) xor C(31) xor D(0) xor D(6);
        NewCRC(4)  := C(28) xor D(3) xor C(27) xor D(4) xor C(26) xor D(5) xor C(24) xor C(30) xor D(1) xor D(7);
        NewCRC(5)  := C(29) xor D(2) xor C(28) xor D(3) xor C(27) xor D(4) xor C(25) xor C(31) xor D(0) xor D(6) xor C(24) xor C(30) xor D(1) xor D(7);
        NewCRC(6)  := C(30) xor D(1) xor C(29) xor D(2) xor C(28) xor D(3) xor C(26) xor D(5) xor C(25) xor C(31) xor D(0) xor D(6);
        NewCRC(7)  := C(31) xor D(0) xor C(29) xor D(2) xor C(27) xor D(4) xor C(26) xor D(5) xor C(24) xor D(7);
        NewCRC(8)  := C(0) xor C(28) xor D(3) xor C(27) xor D(4) xor C(25) xor D(6) xor C(24) xor D(7);
        NewCRC(9)  := C(1) xor C(29) xor D(2) xor C(28) xor D(3) xor C(26) xor D(5) xor C(25) xor D(6);
        NewCRC(10) := C(2) xor C(29) xor D(2) xor C(27) xor D(4) xor C(26) xor D(5) xor C(24) xor D(7);
        NewCRC(11) := C(3) xor C(28) xor D(3) xor C(27) xor D(4) xor C(25) xor D(6) xor C(24) xor D(7);
        NewCRC(12) := C(4) xor C(29) xor D(2) xor C(28) xor D(3) xor C(26) xor D(5) xor C(25) xor D(6) xor C(24) xor C(30) xor D(1) xor D(7);
        NewCRC(13) := C(5) xor C(30) xor D(1) xor C(29) xor D(2) xor C(27) xor D(4) xor C(26) xor D(5) xor C(25) xor C(31) xor D(0) xor D(6);
        NewCRC(14) := C(6) xor C(31) xor D(0) xor C(30) xor D(1) xor C(28) xor D(3) xor C(27) xor D(4) xor C(26) xor D(5);
        NewCRC(15) := C(7) xor C(31) xor D(0) xor C(29) xor D(2) xor C(28) xor D(3) xor C(27) xor D(4);
        NewCRC(16) := C(8) xor C(29) xor D(2) xor C(28) xor D(3) xor C(24) xor D(7);
        NewCRC(17) := C(9) xor C(30) xor D(1) xor C(29) xor D(2) xor C(25) xor D(6);
        NewCRC(18) := C(10) xor C(31) xor D(0) xor C(30) xor D(1) xor C(26) xor D(5);
        NewCRC(19) := C(11) xor C(31) xor D(0) xor C(27) xor D(4);
        NewCRC(20) := C(12) xor C(28) xor D(3);
        NewCRC(21) := C(13) xor C(29) xor D(2);
        NewCRC(22) := C(14) xor C(24) xor D(7);
        NewCRC(23) := C(15) xor C(25) xor D(6) xor C(24) xor C(30) xor D(1) xor D(7);
        NewCRC(24) := C(16) xor C(26) xor D(5) xor C(25) xor C(31) xor D(0) xor D(6);
        NewCRC(25) := C(17) xor C(27) xor D(4) xor C(26) xor D(5);
        NewCRC(26) := C(18) xor C(28) xor D(3) xor C(27) xor D(4) xor C(24) xor C(30) xor D(1) xor D(7);
        NewCRC(27) := C(19) xor C(29) xor D(2) xor C(28) xor D(3) xor C(25) xor C(31) xor D(0) xor D(6);
        NewCRC(28) := C(20) xor C(30) xor D(1) xor C(29) xor D(2) xor C(26) xor D(5);
        NewCRC(29) := C(21) xor C(31) xor D(0) xor C(30) xor D(1) xor C(27) xor D(4);
        NewCRC(30) := C(22) xor C(31) xor D(0) xor C(28) xor D(3);
        NewCRC(31) := C(23) xor C(29) xor D(2);
    
    
        return(NewCRC);
    
      end parallel_crc;

begin
--    process(clk_sig)
--    begin
--        if (clk_sig'event and clk_sig = '1') then
--            if (enable_sig = '1') then
--                -- solamente generamos cuando el bloque de control lo indique
--                if (emitiendo_crc_condicion_1 = '0' or emitiendo_crc_condicion_2 = '0') then
--                --if (emitiendo_crc_condicion_1 = '0' ) then
--                -- calculando CRC
--                    crc <= parallel_crc(crc, mData_sig) XOR X"FFFFFFFF";
--                    --crc <= nextCRC32_D8(mData_sig,crc);
--                    fsc_sig <= "00000000";
                                       
--                    if (mLast_sig = '1') then
--                        emitiendo_crc_condicion_1 <= '1';
--                    else
--                        if (pInterrupt_sig = '1') then
--                            emitiendo_crc_condicion_1 <= '1';
--                            emitiendo_crc_condicion_2 <= '1';
--                        end if;
--                    end if;
                         
--                    if (end_data_gen_sig = '1') then
--                        emitiendo_crc_condicion_2 <= '1';
--                    end if;
--                else
--                -- emitiendo CRC ya calculado (4 octetos)
--                    if (pInterrupt_sig = '1') then
--                        --funcionamineto con interrupcion: calculo mCRC
--                        if (octetos_emitidos = 0 or octetos_emitidos = 1)then
--                            fsc_sig <=  fsc_aux_sig XOR octetos_1_2_XOR;
--                        elsif (octetos_emitidos = 2 or octetos_emitidos = 3)then
--                            fsc_sig <=  fsc_aux_sig XOR octetos_3_4_XOR;
--                        else
--                            fsc_sig <= "00000000";
--                        end if;
--                    else
--                        --funcionamiento normal: sin interrupcion (paquete completo)
--                        fsc_sig <=  fsc_aux_sig;
--                    end if;
--                    octetos_emitidos <= octetos_emitidos + 1;
                                    
--                    if (octetos_emitidos = 3) then
--                    -- finalizacion de CRC
--                        ended_sig <= '1';
--                    end if;
                    
                    
--                end if;
--            else
--                fsc_sig <= "00000000";
--                crc <= "11111111111111111111111111111111";
--                ended_sig <= '0';
--                emitiendo_crc_condicion_1 <= '0';
--                emitiendo_crc_condicion_2 <= '0';   
--                octetos_emitidos <= 0;            
--            end if;
--        end if;
--    end process;
    
-- with octetos_emitidos select
--       fsc_aux_sig <=  crc(31 downto 24) when 0,
--                       crc(23 downto 16) when 1,
--                       crc(15 downto 8)  when 2,
--                       crc(7 downto 0)   when 3,
--                       "00000000"        when others;

    process(clk_sig)
    begin
        if (clk_sig'event and clk_sig = '1') then
            if (enable_sig = '1') then
                if (emitiendo_crc_condicion_1 = '0' or emitiendo_crc_condicion_2 = '0') then
                    Init <= '0';
                    Data_en <= '1';
                                       
                    if (mLast_sig = '1') then
                        emitiendo_crc_condicion_1 <= '1';
                    else
                        if (pInterrupt_sig = '1') then
                            emitiendo_crc_condicion_1 <= '1';
                            emitiendo_crc_condicion_2 <= '1';
                        end if;                                     
                    end if;
                         
                    if (data_gen_sig = '0') then
                        emitiendo_crc_condicion_2 <= '1';
                    end if;
                else
                -- emitiendo CRC ya calculado (4 octetos)
                    if (pInterrupt_sig = '1') then
                        --funcionamineto con interrupcion: calculo mCRC
                        if (octetos_emitidos < 2)then --Default(2)
                            
                        else
                             if (octetos_emitidos = 2) then -- Default(2)
                                CRC_rd <= '1';
                                Data_en <= '0';
                            else
                                Data_en <= '0'; 
                            end if;
                        end if;
                        
                        octetos_emitidos <= octetos_emitidos + 1;
                                            
                        if (octetos_emitidos = 10) then -- Default (10)
                        -- finalizacion de CRC
                            ended_sig <= '1';
                            CRC_rd <= '0';
                            Data_en <= '0';
                            Init <= '1';
                        end if;
                    else
                        --funcionamiento normal: sin interrupcion (paquete completo)
                        if (octetos_emitidos = 1) then
                            CRC_rd <= '1';
                            Data_en <= '0'; 
                            --ended_sig <= '1';
                        end if;
                        
                        if (octetos_emitidos = 2) then
                            Data_en <= '0';  
                            --ended_sig <= '1';                 
                        end if;
                        
                        octetos_emitidos <= octetos_emitidos + 1;
                                            
                        if (octetos_emitidos = 6) then
                        -- finalizacion de CRC
                            ended_sig <= '1';
                            CRC_rd <= '0';
                            Data_en <= '0';
                            Init <= '1';
                        end if;
                    end if;
--                    octetos_emitidos <= octetos_emitidos + 1;
                    
--                    if (octetos_emitidos = 5) then
--                    -- finalizacion de CRC
--                        ended_sig <= '1';
--                        CRC_rd <= '0';
--                        Data_en <= '0';
--                        Init <= '1';
--                    end if;
                    
--                    if (octetos_emitidos > 10) then
--                        ended_sig <= '0';
--                    end if;
                    
                end if;
            else
                Init <= '1';
                CRC_rd <= '0'; 
                emitiendo_crc_condicion_1 <= '0';
                emitiendo_crc_condicion_2 <= '0'; 
                Data_en <= '0';
                ended_sig <= '0';   
                octetos_emitidos <= 0;     
            end if;
        end if;
    end process;


    process (clk_sig) is
    begin
        if rising_edge(clk_sig) then
          if Init = '1' then
            CRC_reg <= X"ffffffff";
          elsif Data_en = '1' then
            CRC_reg <= parallel_crc(CRC_reg, Frame_data);
          elsif CRC_rd = '1' then
            CRC_reg <= CRC_reg(23 downto 0)&X"ff";
          end if;
          
          CRC_reg_not <= not(CRC_reg);
        end if;
    end process;

    process (clk_sig) is
    begin
        if rising_edge(clk_sig) then
          if CRC_rd = '1' then
            if (pInterrupt_sig = '0') then
                CRC_out(7 downto 0) <= not(CRC_reg(24))&not(CRC_reg(25))&not(CRC_reg(26))&not(CRC_reg(27))&not(CRC_reg(28))&not(CRC_reg(29))&not(CRC_reg(30))&not(CRC_reg(31));
            else
                if (octetos_emitidos = 3 or octetos_emitidos = 4)then -- Default(3-4)
                    CRC_out(7 downto 0) <= (not(CRC_reg(24))&not(CRC_reg(25))&not(CRC_reg(26))&not(CRC_reg(27))&not(CRC_reg(28))&not(CRC_reg(29))&not(CRC_reg(30))&not(CRC_reg(31))) XOR "11111111";
                elsif (octetos_emitidos = 5 or octetos_emitidos = 6)then -- Default(5-6)
                    CRC_out(7 downto 0) <= not(CRC_reg(24))&not(CRC_reg(25))&not(CRC_reg(26))&not(CRC_reg(27))&not(CRC_reg(28))&not(CRC_reg(29))&not(CRC_reg(30))&not(CRC_reg(31));
                else
                    CRC_out(7 downto 0) <= not(CRC_reg(24))&not(CRC_reg(25))&not(CRC_reg(26))&not(CRC_reg(27))&not(CRC_reg(28))&not(CRC_reg(29))&not(CRC_reg(30))&not(CRC_reg(31));
                end if;
            end if;
          end if;
        end if;
    end process;


----Asignacion de entradas a seales internas
clk_sig <= clk;
enable_sig <= enable;
--mData_sig <= mData;
mLast_sig <= mLast;
pInterrupt_sig <= pInterrupt;
--end_data_gen_sig <= end_data_gen;
data_gen_sig <= data_gen;
----Asignacion de seales internas a salidas
--fsc <= fsc_sig;
ended <= ended_sig;


Frame_data <= mData;  --! Input data
fsc         <= CRC_out;

end Behavioral;
