----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 02.03.2019 14:34:14
-- Design Name: 
-- Module Name: maquina_estados_generador - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity maquina_estados_generador is
    Port ( eTx_OK : in STD_LOGIC;
           pTx_OK : in STD_LOGIC;
           rst : in STD_LOGIC;
           clk : in STD_LOGIC;
           enable : in STD_LOGIC;
           sel : in STD_LOGIC_VECTOR (1 downto 0);
           eGen : out STD_LOGIC;
           pGen : out STD_LOGIC;
           enable_DataGen : out STD_LOGIC);
end maquina_estados_generador;

architecture Behavioral of maquina_estados_generador is

    type estados is (espera, gen_Express, gen_Preemtable, gen_Stop);
    
    -- seales para estados
    signal ea : estados;    -- seal para estado actual
    signal es : estados;    -- seal para estado siguiente
    
    -- seales internas para entradas
    signal eTx_OK_sig : STD_LOGIC := '0';
    signal pTx_OK_sig : STD_LOGIC := '0';
    signal rst_sig : STD_LOGIC := '0';
    signal clk_sig : STD_LOGIC := '0';
    signal enable_sig : STD_LOGIC := '0';
    signal sel_sig : STD_LOGIC_VECTOR (1 downto 0) := "00";
    
    --seales internas para salidas
    signal eGen_sig : STD_LOGIC := '0';
    signal pGen_sig : STD_LOGIC := '0';
    signal enable_DataGen_sig : STD_LOGIC := '0';
        
begin

-- PENDIENTE.
-- CONTROLAR REINICIO DE ESTADO AUTOMATICO SI FINALIZA LA GENERACION Y NO SE RECIBE ACK
--

------------------------------------------------------------
------------	MAQUINA ESTADOS: FLIP-FLOPS	---------------
------------------------------------------------------------
    process (clk_sig)
    begin
         if (clk_sig'event and clk_sig='1') then
             if (rst_sig = '1') then
             -- inicializacion a estado inicial
                 ea <= espera;
             else
             -- funcionamiento normal, pasamos a siguiente estado
                ea <= es;
             end if;        
        end if;
    end process;
    
------------------------------------------------------------
----------	MAQUINA ESTADOS: CONTROL ENTRADAS	------------
------------------------------------------------------------
    process (clk_sig, ea)
    begin
        if (clk_sig'event and clk_sig='1') then
        -- seleccion del estado siguiente en funcion del estado actual y de las entradas en dicho momento
            case ea is     
                when espera =>
                -- estado inicial (1)
                    if (sel_sig = "01") then
                    -- seleccion de generacion de Preemptable Packet
                        es <= gen_Preemtable;
                    elsif (sel_sig = "10" or sel_sig = "11") then
                    -- seleccion de generacion de Express Packet
                        es <= gen_Express;
                    else
                        es <= espera;
                    end if;
                    
                when gen_Express =>
                -- estado de generacion de paquete Express (2)
                    if (eTx_OK_sig = '1') then
                    -- transmision de ePacket correcta
                        if (sel_sig = "01") then
                        -- seleccion de generacion de Preemptable Packet
                            es <= gen_Preemtable;
                        elsif (sel_sig = "10" or sel_sig = "11") then
                        -- seleccion de generacion de Express Packet
                            es <= gen_Express;
                        else
                            es <= gen_Stop;
                        end if;
                    else
                    -- debe seguir transmitiendo hasta fin de Express Packet
                        es <= gen_Express;
                    end if;
                
                when gen_Preemtable =>
                -- estado de generacion de paquete Preemtable (3)
                    if (sel_sig = "10" or sel_sig = "11") then
                    -- llega paquete Express y se detiene trasnmision de paquete Preemptable
                        es <= gen_Express; -- PENDIENTE: CONTROLAR ALMACENADO DE DATOS DE PPACKET
                    else
                        if (pTx_OK_sig = '1') then
                        -- transmision de pPacket correcta       
                            if (sel_sig = "01") then
                            -- seleccion de generacion de Preemptable Packet
                                es <= gen_Preemtable;
                            elsif (sel_sig = "10" or sel_sig = "11") then
                            -- seleccion de generacion de Express Packet
                                es <= gen_Express;
                            else
                                es <= gen_Stop;
                            end if;                      
                        end if;
                    end if;
                when gen_Stop =>
                    es <= espera;
            end case;
        end if;        
    end process;
	

------------------------------------------------------------
----------	MAQUINA ESTADOS: CONTROL SALIDAS	------------
------------------------------------------------------------
	WITH ea SELECT
	   enable_DataGen_sig  <=  '0' when gen_Stop,
	                           '1' when others;
	
	WITH ea SELECT
       eGen_sig            <=   '1' when gen_Express,
                                '0' when others;
		
						
	WITH ea SELECT
		pGen_sig           <=   '1' when gen_Preemtable,
                                '0' when others;
					
						
--------------------------------------------------------------------------------
--- asignacion de valores de seales internas mediante los puertos    ----------
--------------------------------------------------------------------------------		
eTx_OK_sig <= eTx_OK;
pTx_OK_sig <= pTx_OK;
rst_sig <= rst;
clk_sig <= clk;
enable_sig <= enable;
sel_sig <= sel;

--------------------------------------------------------------------------------
--- asignacion de valores de los puertos mediante las seales internas    ------
--------------------------------------------------------------------------------
eGen <= eGen_sig;
pGen <= pGen_sig;
enable_DataGen <= enable_DataGen_sig;



end Behavioral;
