----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 01.03.2019 20:02:28
-- Design Name: 
-- Module Name: contador_dsc - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity number_generator is
    Port ( enable : in STD_LOGIC;                       -- generating del bloque contador
           last : in STD_LOGIC;                         -- last del bloque contador
           clk : in STD_LOGIC;
           rst : in STD_LOGIC;
           mData : out STD_LOGIC_VECTOR (7 downto 0);   -- datos del paquete generado
           mValid : out STD_LOGIC;                      -- indicador de paquete generado listo
           mLast : out STD_LOGIC;                       -- indicador de ultimo paquete generado listo
           mLength : in STD_LOGIC_VECTOR (7 downto 0)); -- tamao del paquete generado
end number_generator;

architecture Behavioral of number_generator is

    -- seales internas para las entradas del bloque
    signal clk_sig : std_logic := '0';
    signal rst_sig : std_logic := '0';
    signal enable_sig : std_logic := '0';
    signal last_sig : std_logic := '0';
    signal packet_length_sig: std_logic_vector (7 downto 0) := "00000000";

    -- seales internas para las salidas del bloque
    signal mValid_sig: std_logic := '0';
    signal mLast_sig : std_logic := '0';
    signal mData_sig: std_logic_vector (7 downto 0) := "00000000";
    
    -- seales internas para la funcion del generador
    signal length_val: integer range 0 to 254 := 0;                     -- Valor del tamao a generar 
    constant data_val: std_logic_vector (7 downto 0) :=  "11111111";    -- Valor de datos de paquetes constante
    constant data_zero: std_logic_vector (7 downto 0) :=  "00000000";   -- Valor inicial de datos de paquetes
    
    -- seales internas para control adicional
    signal enable_aux_inicio: std_logic := '0';                                    -- Seal para control de emision de primer paquete
    signal enable_aux_fin: std_logic := '0';                                       -- Seal para control de emision de ultimo paquete

    -- seales y constantes para valores de datos de salida
    signal dato_n : integer range 0 to 7 := 0;
    signal dato_sig : std_logic_vector (7 downto 0) :=  "00000000";
    constant dato_0 : std_logic_vector (7 downto 0) :=  "00000001";
    constant dato_1 : std_logic_vector (7 downto 0) :=  "00000010";
    constant dato_2 : std_logic_vector (7 downto 0) :=  "00000101";
    constant dato_3 : std_logic_vector (7 downto 0) :=  "00001010";
    constant dato_4 : std_logic_vector (7 downto 0) :=  "00010101";
    constant dato_5 : std_logic_vector (7 downto 0) :=  "00101010";
    constant dato_6 : std_logic_vector (7 downto 0) :=  "01010101";
    constant dato_7 : std_logic_vector (7 downto 0) :=  "10101010";
    
    
begin


process (clk_sig)
begin
    if (clk_sig'event and clk_sig='1') then
    -- si tiene lugar evento de reloj y en pulso ascendente
        if (rst_sig = '1') then
        -- en caso de reset (inicializacion)
                length_val <= 0;
                mData_sig <= (others=>'0');
                mValid_sig <= '0';
                mLast_sig <= '0';
                enable_aux_inicio <= '0';  
                enable_aux_fin <= '0';      
        else
        -- en caso de funcionamiento normal 
            if (enable_sig = '1') then
            -- habilitacion activada
                length_val <= to_integer(unsigned(packet_length_sig));
                mData_sig <= (others=>'0');                  
                
                if (dato_n = 7) then        --recorremos de forma ciclina los n dato_n
                    dato_n <= 0;
                else
                    dato_n <= (dato_n + 1);
                end if;
                mData_sig <= dato_sig;      --asignamos valor correspondiente al ciclo de reloj actual
                
                --mData_sig <= (data_zero((mData_sig'length - 1) downto (length_val))) & (data_val((length_val - 1) downto 0));---- rellenamos los datos por la derecha hasta tener la longitud a generar
                if (last = '1') then
                    --last_in <= '1';
                    enable_aux_fin <= '1';
                else
                    mLast_sig <= '0';
                    enable_aux_fin <= '0';
                end if;
                    if(enable_aux_inicio = '0') then
                    -- retrasamos un ciclo el tick de paquete valido para sincronizacion adecuada
                        enable_aux_inicio <= '1';     
                    else
                        mValid_sig <= '1';        
                    end if;   
            else
                if(enable_aux_fin = '1')then
                -- control de la emision de ultimo paquete tras finalizacion de habilitador
                    length_val <= to_integer(unsigned(packet_length_sig));
                    mData_sig <= (others=>'0');                  

                    if (dato_n = 7) then        --recorremos de forma ciclina los n dato_n
                        dato_n <= 0;
                    else
                        dato_n <= (dato_n + 1);
                    end if;
                    mData_sig <= dato_sig;      --asignamos valor correspondiente al ciclo de reloj actual

                    --mData_sig <= (data_zero((mData_sig'length - 1) downto (length_val))) & (data_val((length_val - 1) downto 0));-- rellenamos los datos por la derecha hasta tener la longitud a generar
                    enable_aux_fin <= '0';
                    mLast_sig <= '1';
                    mValid_sig <= '1';        
                
                else
                    if(mLast_sig = '1') then
                    -- si ya hemos enviado el ultimo paquete, inicializamos
                        length_val <= 0;
                        mData_sig <= (others=>'0');
                        mValid_sig <= '0';
                        mLast_sig <= '0';
                        enable_aux_inicio <= '0';
                        enable_aux_fin <= '0';    
                    end if;
                end if;    
            end if;
        end if;
    end if;
end process;


--Seleccion de datos a generar
with dato_n select
    dato_sig <= dato_0 when 0,
                dato_1 when 1,
                dato_2 when 2,
                dato_3 when 3,
                dato_4 when 4,
                dato_5 when 5,
                dato_6 when 6,
                dato_7 when 7;

-- asignacion de valores de seales internas mediante los puertos
clk_sig <= clk;
rst_sig <= rst;
enable_sig <= enable;
last_sig <= last;
packet_length_sig <= mLength;

-- asignacion de valores a los puertos mediante las seales internas
mValid <= mValid_sig; 
mLast <= mLast_sig;
mData <= mData_sig;


end Behavioral;