----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 10.04.2019 16:19:42
-- Design Name: 
-- Module Name: preamble_generator - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity preamble_generator is
    Port ( clk : in STD_LOGIC;
           enable_generate : in STD_LOGIC;
           pInterrupt : in STD_LOGIC;
           eGen : in STD_LOGIC;
           pGen : in STD_LOGIC;
           preamble : out STD_LOGIC_VECTOR (7 downto 0);
           ended : out STD_LOGIC;
           valid : out STD_LOGIC);
end preamble_generator;

architecture Behavioral of preamble_generator is

    --seales internas para entradas
    signal clk_sig :  STD_LOGIC := '0';
    signal enable_generate_sig :  STD_LOGIC := '0';
    signal pInterrupt_sig :  STD_LOGIC := '0';
    signal eGen_sig :  STD_LOGIC := '0';
    signal pGen_sig :  STD_LOGIC := '0';
    --seales internas para salidas
    signal preamble_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
    signal ended_sig :  STD_LOGIC := '0';
    signal valid_sig :  STD_LOGIC := '0';
    
    --seales internas relativas a la fragmentacion de paquetes
    --signal n_frag : integer range 0 to 255 := 0;
    
    --seales internas para control de interrupcion de paquetes
     signal hay_interrupcion : STD_LOGIC := '0';
     --signal hay_interrupcion_siguiente : STD_LOGIC := '0';
     
    --seales internas para gestion de inicializacion de seales
    signal inicializado : STD_LOGIC := '0';
    signal smd_sig : STD_LOGIC_VECTOR  (7 downto 0)  := "11010101";  --0xD5
    signal smd_sx_sig : STD_LOGIC_VECTOR  (7 downto 0)  := "11100110";  --0xE6
    signal smd_cx_sig : STD_LOGIC_VECTOR  (7 downto 0)  := "01100001";  --0x61
    signal frag_count_sig : STD_LOGIC_VECTOR  (7 downto 0)  := "01100111";  --0xE6
    signal smd_generado :  STD_LOGIC := '0';
    
    --seales internas para control de numero de octetos en preamble
    signal preamble_octets : integer range 0 to 7 := 0; --Puede tomar valor de 7 para paquete Express, Preempatble completo o 1er fragmento y 6 para Preempatable continuacion
   
    --seales internas para control de secuencia de generacion de SFD -> SMD_Sx y SMD_Cx (Conteo ciclico: x={0,1,2,3,0,1,2,3,0,1...})
    signal smd_sx_count : integer range 0 to 3 := 0;
    signal smd_cx_count : integer range 0 to 3 := 0;
    signal frag_count   : integer range 0 to 254 := 0;
    
    -- constantes para generacion de cabecera
    constant unidad_0to7 : integer range 0 to 7 := 1;
    constant unidad_0to3 : integer range 0 to 3 := 1;
    constant unidad_0to254 : integer range 0 to 254 := 1;
    
    constant smd_X0_count : integer range 0 to 3 := 0;
    constant smd_X1_count : integer range 0 to 3 := 1;
    constant smd_X2_count : integer range 0 to 3 := 2;
    constant smd_X3_count : integer range 0 to 3 := 3;
    
    constant numero_octetos_7 : integer range 0 to 7 := 7;
    constant numero_octetos_6 : integer range 0 to 7 := 6;
    
    
    --constantes no invertidas
--    constant no_val :       STD_LOGIC_VECTOR  (7 downto 0)  := "00000000";  --0x00
--    constant preamble_val:  STD_LOGIC_VECTOR  (7 downto 0)  := "10101010";  --0x55
    
--    constant sfd_val:       STD_LOGIC_VECTOR  (7 downto 0)  := "11010101";  --0xD5
    
--    constant smd_e:         STD_LOGIC_VECTOR  (7 downto 0)  := "11010101";  --0xD5
    
--    constant smd_s0:        STD_LOGIC_VECTOR  (7 downto 0)  := "11100110";  --0xE6
--    constant smd_s1:        STD_LOGIC_VECTOR  (7 downto 0)  := "01001100";  --0x4C
--    constant smd_s2:        STD_LOGIC_VECTOR  (7 downto 0)  := "01111111";  --0x7F
--    constant smd_s3:        STD_LOGIC_VECTOR  (7 downto 0)  := "10110011";  --0xB3
    
--    constant smd_c0:        STD_LOGIC_VECTOR  (7 downto 0)  := "01100001";  --0x61
--    constant smd_c1:        STD_LOGIC_VECTOR  (7 downto 0)  := "01010010";  --0x52
--    constant smd_c2:        STD_LOGIC_VECTOR  (7 downto 0)  := "10011110";  --0x9E
--    constant smd_c3:        STD_LOGIC_VECTOR  (7 downto 0)  := "00101010";  --0x2A
    
--    constant frag_count_0:  STD_LOGIC_VECTOR  (7 downto 0)  := "11100110";  --0xE6
--    constant frag_count_1:  STD_LOGIC_VECTOR  (7 downto 0)  := "01001100";  --0x4C
--    constant frag_count_2:  STD_LOGIC_VECTOR  (7 downto 0)  := "01111111";  --0x7F
--    constant frag_count_3:  STD_LOGIC_VECTOR  (7 downto 0)  := "10110011";  --0xB3
    
    --constantes invertidas
    constant no_val :       STD_LOGIC_VECTOR  (7 downto 0)  := "00000000";  --0x00
    constant preamble_val:  STD_LOGIC_VECTOR  (7 downto 0)  := "10101010";  --0x55
    
    constant sfd_val:       STD_LOGIC_VECTOR  (7 downto 0)  := "10101011";  --0xD5
    
    constant smd_e:         STD_LOGIC_VECTOR  (7 downto 0)  := "10101011";  --0xD5
    
    constant smd_s0:        STD_LOGIC_VECTOR  (7 downto 0)  := "01100111";  --0xE6
    constant smd_s1:        STD_LOGIC_VECTOR  (7 downto 0)  := "00110010";  --0x4C
    constant smd_s2:        STD_LOGIC_VECTOR  (7 downto 0)  := "11111110";  --0x7F
    constant smd_s3:        STD_LOGIC_VECTOR  (7 downto 0)  := "11001101";  --0xB3
    
    constant smd_c0:        STD_LOGIC_VECTOR  (7 downto 0)  := "10000110";  --0x61
    constant smd_c1:        STD_LOGIC_VECTOR  (7 downto 0)  := "01001010";  --0x52
    constant smd_c2:        STD_LOGIC_VECTOR  (7 downto 0)  := "01111001";  --0x9E
    constant smd_c3:        STD_LOGIC_VECTOR  (7 downto 0)  := "01010100";  --0x2A
    
    constant frag_count_0:  STD_LOGIC_VECTOR  (7 downto 0)  := "01100111";  --0xE6
    constant frag_count_1:  STD_LOGIC_VECTOR  (7 downto 0)  := "00110010";  --0x4C
    constant frag_count_2:  STD_LOGIC_VECTOR  (7 downto 0)  := "11111110";  --0x7F
    constant frag_count_3:  STD_LOGIC_VECTOR  (7 downto 0)  := "11001101";  --0xB3
    
    
begin

    process (clk_sig)
    begin
        if (clk_sig'event and clk_sig='1') then 
            
            if (pInterrupt_sig='1') then
                    if (hay_interrupcion ='0' ) then
                        --si tiene lugar una interrupcion, ponemos el flag correspondiente a 1 para no perder la condicion durante la generacion de datos
                        hay_interrupcion <= '1';
                    end if;
            end if;
                
            if (enable_generate_sig = '1') then
            -- solamente generamos cuando el bloque de control lo indique
          
                    if (hay_interrupcion = '1') then
                    -- ha habido interrupcion de paquete Preemtable
                        if (eGen_sig = '1') then
                            if (inicializado = '0') then
                                -- inicializamos valores del preamble de ePacket
                                preamble_octets <= 0;   --para un ePacket se deben transmitir 7 octetos "preamble_val"
                                smd_sig <= smd_e;       --asiganmos al valor de SFD el valor SMD-E correspondiente a ePacket    
                                
                                --hay_interrupcion_siguiente <= '1';  -- se mantiene el paquete interrumpido en espera para generar paquete Express
                                --hay_interrupcion <= hay_interrupcion_siguiente;
                                
                                inicializado <= '1';   --ponemos flag a 1 para evitar volver a entrar
                                ended_sig <= '0';
                                
                            else                                             
                                if (ended_sig = '0') then
                                    valid_sig <= '1';
                                    -- se genera la cabecera propia de un paquete Express
                                    if (preamble_octets < numero_octetos_7) then
                                        --generamos bytes de "preamble_val" hasta acabar con el total 
                                        preamble_sig <= preamble_val;
                                        preamble_octets <= (preamble_octets + unidad_0to7);
                                    
                                    else
                                        --generamos ultimo octeto correspondiente al SFD
                                        preamble_sig <= smd_e;
                                        ended_sig <= '1';
                                    end if;
                                else
                                    preamble_sig <= no_val;
                                    valid_sig <= '0';
                                end if;
                            end if;
                            
                        elsif (eGen_sig = '0' AND pGen_sig = '1') then 
                            if (inicializado = '0') then
                                -- inicializamos valores del preamble de pPacket (continuacion de fragmento)
                                preamble_octets <= 0;               --para un pPacket (cont) se deben transmitir 6 octetos "preamble_val"
                                
                                --recuperamos valor de ultima transmision de pPacket
    --                            if (smd_cx_count = 0) then
    --                                smd_cx_count <= 3;      --ciclico
    --                            else
    --                                smd_cx_count <= (smd_cx_count - unidad_0to3);
    --                            end if;
    
                                --smd_sig <= smd_cx_sig;
                                
                                --hay_interrupcion_siguiente <= '1';  -- se mantiene el paquete interrumpido en espera para generar paquete Preemptable continuation                
                                --hay_interrupcion <= hay_interrupcion_siguiente;
                                
                                inicializado <= '1';   --ponemos flag a 1 para evitar volver a entrar
                                smd_generado <= '0';
                                ended_sig <= '0';
                            else
                                smd_sig <= smd_cx_sig;  
                                if (ended_sig = '0') then
                                    valid_sig <= '1';
                                    -- se genera la cabecera propia de un paquete Preemptable
                                    if (preamble_octets < numero_octetos_6) then
                                        --generamos bytes de "preamble_val" hasta acabar con el total 
                                        preamble_sig <= preamble_val;
                                        preamble_octets <= (preamble_octets + unidad_0to7);
                                    
                                    else
                                        if (smd_generado = '0') then
                                            --generamos octeto correspondiente al SFD
                                            preamble_sig <= smd_cx_sig;                                 
                                            smd_generado <= '1';
                                        else
                                            --generamos ultimo octeto del preamble correspondiente al frag_count
                                            
                                            --recuperamos secuencia de siguiente pPacket que se va a transmitir
    --                                        if (smd_cx_count = 3) then
    --                                            smd_cx_count <= 0;              --ciclico
    --                                        else
    --                                            smd_cx_count <= (smd_cx_count + unidad_0to3); 
    --                                        end if;
                                            
                                            --preamble_sig <= std_logic_vector(to_unsigned(frag_count,preamble_sig'length));
                                            preamble_sig <= frag_count_sig;
                                            frag_count <= (frag_count + unidad_0to254);
                                            ended_sig <= '1';
                                            
                                            hay_interrupcion <= '0';  -- se libera el paquete interrumpido en espera hasta proxima interrupcion
                                            --hay_interrupcion <= hay_interrupcion_siguiente;
                                        end if;
                                    end if;  
                                else
                                    preamble_sig <= no_val; 
                                    valid_sig <= '0';  
                                end if;           
                            end if;
                        else
                        -- esperamos hasta que llegue informacion de paquete Preemtable para continuar
                        
                        end if;
               ---------------------------------------------------------------------------------------------------------------------------------- 
                    else
                    -- no ha habido interrupcion anterior
                        if (eGen_sig = '1') then      
                            if (inicializado = '0') then
                                -- inicializamos valores del preamble de ePacket
                                preamble_octets <= 0;   --para un ePacket se deben transmitir 7 octetos "preamble_val"
                                smd_sig <= smd_e;       --asiganmos al valor de SFD el valor SMD-E correspondiente a ePacket    
                                
                                --hay_interrupcion_siguiente <= '0';  -- se libera el paquete interrumpido en espera hasta proxima interrupcion
                                --hay_interrupcion <= hay_interrupcion_siguiente;
                                
                                inicializado <= '1';   --ponemos flag a 1 para evitar volver a entrar
                                
                                frag_count <= 0; --inicializamos a cero ya que no ha habido ninguna interrupcion
                                ended_sig <= '0';
                            else
                                if (ended_sig = '0') then
                                    valid_sig <= '1';
                                    -- se genera la cabecera propia de un paquete Express
                                    if (preamble_octets < numero_octetos_7) then
                                        --generamos bytes de "preamble_val" hasta acabar con el total 
                                        preamble_sig <= preamble_val;
                                        preamble_octets <= (preamble_octets + unidad_0to7);
                                    
                                    else
                                        --generamos ultimo octeto correspondiente al SFD
                                        preamble_sig <= smd_sig;
                                        ended_sig <= '1';
                                    end if;
                                else
                                    --ya se ha terminado de generar
                                    preamble_sig <= no_val;
                                    valid_sig <= '0';
                               end if;     
                            end if;
                             
                        elsif (eGen_sig = '0' AND pGen_sig = '1') then
                            
                            -- se genera la cabecera propia de un paquete Preemtable (completo o primer fragmento)
                            if (inicializado = '0') then
                                -- inicializamos valores del preamble de pPacket (completo o primer fragmento)
                                preamble_octets <= 0;               --para un pPacket (comp / 1er freg) se deben transmitir 7 octetos "preamble_val"
                                                         
                                --incrementamos valor de secuencia de transmision de pPacket para paquetes completos o primeros fragmentos
                                if (smd_sx_count = 3) then
                                    smd_sx_count <= 0;              --ciclico
                                else
                                    smd_sx_count <= (smd_sx_count + unidad_0to3);   
                                end if;
                                --incrementamos valor de secuencia de transmision de pPacket para paquetes de continuacion tras interrupcion
                                if (smd_cx_count = 3) then
                                    smd_cx_count <= 0;              --ciclico
                                else
                                    smd_cx_count <= (smd_cx_count + unidad_0to3);   
                                end if;
                                
                                --smd_sig <= smd_sx_sig; 
         
                                --hay_interrupcion_siguiente <= '0';  -- se quita el flag de paquete interrumpido
                                --hay_interrupcion <= hay_interrupcion_siguiente;
                                
                                inicializado <= '1';   --ponemos flag a 1 para evitar volver a entrar
                                frag_count <= 0; --inicializamos a cero ya que no ha habido ninguna interrupcion
                                ended_sig <= '0';
                            else      
                                smd_sig <= smd_sx_sig; 
                                if (ended_sig = '0') then
                                    valid_sig <= '1';                  
                                    -- se genera la cabecera propia de un paquete Preemptable
                                    if (preamble_octets < numero_octetos_7) then
                                        --generamos bytes de "preamble_val" hasta acabar con el total 
                                        preamble_sig <= preamble_val;
                                        preamble_octets <= (preamble_octets + unidad_0to7);
                                    
                                    else
                                        if (smd_generado = '0') then
                                            --generamos octeto correspondiente al SFD
                                            preamble_sig <= smd_sig;
                                            ended_sig <= '1';                            
                                        end if;                          
                                    end if;  
                                else
                                    preamble_sig <= no_val;  
                                    valid_sig <= '0'; 
                                end if;  
                            end if;
                            
                        else
                        -- esperamos hasta que llegue nuevo paquete para continuar
                            frag_count <= 0; 
                        end if;
                    end if;   
            else
            -- si no esta habilitado el bloque la salida sera nula
                preamble_sig <= no_val;
                inicializado <= '0';
                ended_sig <= '0';
            end if;
        end if; 
             
    end process;

    
       

    with smd_sx_count select            --asignamos a SFD el SMD-Sx correspondiente de la continuacion al ultimo pPacket
            smd_sx_sig <=   smd_s0 when smd_X0_count,
                            smd_s1 when smd_X1_count,
                            smd_s2 when smd_X2_count,
                            smd_s3 when smd_X3_count;
    
    with smd_cx_count select            --asignamos a SFD el SMD-Cx correspondiente de la continuacion al ultimo pPacket
            smd_cx_sig <=   smd_c0 when smd_X0_count,
                            smd_c1 when smd_X1_count,
                            smd_c2 when smd_X2_count,
                            smd_c3 when smd_X3_count;
                            
    with frag_count select            --asignamos a SFD el frag_count_X correspondiente de la continuacion al ultimo pPacket
                frag_count_sig <=   frag_count_0 when 0,
                                    frag_count_1 when 1,
                                    frag_count_2 when 2,
                                    frag_count_3 when 3,
                                    "01100111" when others;


--Asignacion de seales para registros internos
--hay_interrupcion <= hay_interrupcion_siguiente;

--Asignacion de entradas a seales internas
clk_sig <= clk;
enable_generate_sig <= enable_generate;
pInterrupt_sig <= pInterrupt;
eGen_sig <= eGen;
pGen_sig <= pGen;
--Asignacion de seales internas a salidas
preamble <= preamble_sig;
ended <= ended_sig;
valid <= valid_sig;

end Behavioral;
