----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 15.04.2019 20:36:57
-- Design Name: 
-- Module Name: select_generating_block - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity select_generating_block is
    Port ( clk : in STD_LOGIC;
           rst : in STD_LOGIC;
           mGen : in STD_LOGIC;
           endBlock : in STD_LOGIC_VECTOR (4 downto 0);
           pInterrupt : in STD_LOGIC;
           header_gen : out STD_LOGIC;
           data_gen : out STD_LOGIC;
           selBlock : out STD_LOGIC_VECTOR (3 downto 0);
           fin_gen : out STD_LOGIC;
           valid : out STD_LOGIC);
end select_generating_block;

architecture Behavioral of select_generating_block is


    type estados is (espera, gen_Preamble, gen_FinPreamble_ToPayload, gen_Payload_Header, gen_Payload_Data, gen_FSC, gen_Gap, fin_Ok, fin_Interr);
        
    -- seales para estados
    signal ea : estados;    -- seal para estado actual
    signal es : estados;    -- seal para estado siguiente
    
    --seales internas para entradas
    signal clk_sig :  STD_LOGIC := '0';
    signal rst_sig :  STD_LOGIC := '0';
    signal mGen_sig :  STD_LOGIC := '0';
    signal endBlock_sig : STD_LOGIC_VECTOR (4 downto 0) := "00000";
    --seales internas para salidas
    signal header_gen_sig   : STD_LOGIC := '0';
    signal data_gen_sig     : STD_LOGIC := '0';
    signal selBlock_sig     : STD_LOGIC_VECTOR (3 downto 0) := "0000";
    signal pInterrupt_sig   : STD_LOGIC := '0';
    signal fin_gen_sig      : STD_LOGIC := '0';
    signal valid_sig      : STD_LOGIC := '0';
    
    
    -- constantes para valores de entrada
    constant no_end_gen                 : STD_LOGIC_VECTOR (4 downto 0)   := "00000";
    constant ended_preamble_gen         : STD_LOGIC_VECTOR (4 downto 0)   := "10000";
    constant ended_payload_h_gen        : STD_LOGIC_VECTOR (4 downto 0)   := "01000";
    constant ended_payload_d_gen        : STD_LOGIC_VECTOR (4 downto 0)   := "00100";
    constant ended_payload_h_d_gen      : STD_LOGIC_VECTOR (4 downto 0)   := "01100";
    constant ended_fsc_gen              : STD_LOGIC_VECTOR (4 downto 0)   := "00010";
    constant ended_payl_h_d_fsc_gen     : STD_LOGIC_VECTOR (4 downto 0)   := "01110";
    constant ended_pre_payl_h_d_fsc_gen : STD_LOGIC_VECTOR (4 downto 0)   := "11110";
    constant ended_gap_gen              : STD_LOGIC_VECTOR (4 downto 0)   := "00001";
    
    -- constantes para valores de salida
    constant no_select_gen                      : STD_LOGIC_VECTOR (3 downto 0)   := "0000";
    constant select_preamble_gen                : STD_LOGIC_VECTOR (3 downto 0)   := "1000";
    constant select_preamble_payload_fsc_gen    : STD_LOGIC_VECTOR (3 downto 0)   := "1110";
    constant select_payload_gen                 : STD_LOGIC_VECTOR (3 downto 0)   := "0100";
    constant select_fsc_gen                     : STD_LOGIC_VECTOR (3 downto 0)   := "0010";
    constant select_payload_fsc_gen             : STD_LOGIC_VECTOR (3 downto 0)   := "0110"; --para el correcto calculo del fsc se debe realiar simultaneamente generacion y checksum
    constant select_gap_gen                     : STD_LOGIC_VECTOR (3 downto 0)   := "0001";
    
    --seales internas para mantenimiento de estado durante ciclos de reloj
    --signal numero_ciclos_extra_preamble : integer range 0 to 2 := 0;
    --signal segundo_fsc : STD_LOGIC := '0';
    
begin

------------------------------------------------------------
------------	MAQUINA ESTADOS: FLIP-FLOPS	---------------
------------------------------------------------------------
    process (clk_sig)
    begin
         if (clk_sig'event and clk_sig='1') then
            if (rst_sig = '1') then
                 -- inicializacion a estado inicial
                     ea <= espera;
             else
                -- funcionamiento normal, pasamos a siguiente estado
                ea <= es;
            end if;        
        end if;
    end process;
    
------------------------------------------------------------
----------	MAQUINA ESTADOS: CONTROL ENTRADAS	------------
------------------------------------------------------------
    process (clk_sig, ea)
    begin
        if (clk_sig'event and clk_sig='1') then
        -- seleccion del estado siguiente en funcion del estado actual y de las entradas en dicho momento
            case ea is     
                when espera =>
                -- estado inicial (1)
                    if (mGen_sig = '1') then
                    -- pasamos a generar Preamble de paquete
                        es <= gen_Preamble;
                    --else  
                        --es <= ea;
                    end if;
                
                when gen_Preamble =>
                -- generamos Preamble hasta que el bloque indique fin de generacion (2)
                    if (endBlock_sig = ended_preamble_gen) then
                    -- pasamos a generar Payload_Header de paquete
                        es <= gen_FinPreamble_ToPayload;
                    --else
                        --es <= ea;
                    end if;
                    
                when gen_FinPreamble_ToPayload =>
                -- generamos Preamble hasta que el bloque indique fin de generacion (2)
                        es <= gen_Payload_Header;
 
                    
                when gen_Payload_Header =>
                -- generamos Payload_Header hasta que el bloque indique fin de generacion (3)
                    if (endBlock_sig = ended_payload_h_gen) then
                    -- pasamos a generar Payload_Data de paquete
                        es <= gen_Payload_Data;
                    --else
                       -- es <= ea;
                    end if;
                    
                    
                when gen_Payload_Data =>
                -- generamos Payload_Data hasta que el bloque indique fin de generacion (4)
                    if (endBlock_sig = ended_payload_d_gen or endBlock_sig = ended_payload_h_d_gen) then
                    -- pasamos a generar FSC de paquete
                        es <= gen_FSC;
                    else
                        if (pInterrupt_sig = '1') then
                            es <= gen_FSC;  --generamos mCRC
                        --else
                            --es <= ea;
                        end if;
                    end if;
                    --PENDIENTE:
                    -- si tiene lugar una interrupcion se pasa al estado de finaliacion con interrupcion
                    -- se debe controlar que se generan los datos suficientes y que se emite el CRC al finalizar
                    -- tras finalizar comenzar de nuevo la secuencia para ePacket                
                when gen_FSC =>
                -- estado de generacion de informacion de verificacion de trama (5)
                    if (endBlock_sig = ended_fsc_gen or endBlock_sig = ended_payl_h_d_fsc_gen or endBlock_sig = ended_pre_payl_h_d_fsc_gen ) then
                        es <= gen_GAP;
                    --else
                       -- es <= ea;
                    end if;
                    
                when gen_Gap =>
                -- estado de generacion de gap entre tramas (6)
                    if (pInterrupt_sig = '1') then
                        es <= fin_Interr;  --generamos mCRC
                    else
                        es <= fin_Ok;
                    end if;
                    
                when fin_Ok =>
                -- estado de finalizacion de la generacion de trama correcta (7)
                    es <= espera;
                    
                when fin_Interr =>
                -- estado de finalizacion de la generacion de trama con interrupcion (8)
                    es <= espera;            
                
                
            end case;
        end if;      
    end process;
	

------------------------------------------------------------
----------	MAQUINA ESTADOS: CONTROL SALIDAS	------------
------------------------------------------------------------
	WITH ea SELECT
        header_gen_sig <=   '1' when gen_Preamble,
                            '1' when gen_FinPreamble_ToPayload,
                            '1' when gen_Payload_Header,
                            '0' when others;
	
	WITH ea SELECT
        data_gen_sig <=     '1' when gen_Payload_Data,
                            '0' when others;
		
						
	WITH ea SELECT
        selBlock_sig <=     select_preamble_gen             when gen_Preamble,
                            select_preamble_payload_fsc_gen when gen_FinPreamble_ToPayload,
                            --select_payload_gen  when gen_Payload_Header,
                            --select_payload_gen  when gen_Payload_Data,
                            --select_fsc_gen      when gen_FSC,
                            select_payload_fsc_gen          when gen_Payload_Header,
                            select_payload_fsc_gen          when gen_Payload_Data,
                            select_fsc_gen                  when gen_FSC,
                            select_gap_gen                  when gen_GAP,
                            no_select_gen                   when others;
                            
                            
    WITH ea SELECT
        fin_gen_sig <=      '1' when fin_Ok,
                            '1' when fin_Interr,
                            '0' when others;
                            
    WITH ea SELECT
        valid_sig <=        '0' when espera,
                            '0' when gen_Preamble,
                            '0' when gen_GAP,
                            '0' when fin_Ok,
                            '0' when fin_Interr,
                            '1' when others;
					
						
--------------------------------------------------------------------------------
--- asignacion de valores de seales internas mediante los puertos    ----------
--------------------------------------------------------------------------------		
clk_sig <= clk;
rst_sig <= rst;
mGen_sig <= mGen;
endBlock_sig <= endBlock;
pInterrupt_sig <= pInterrupt;
--------------------------------------------------------------------------------
--- asignacion de valores de los puertos mediante las seales internas    ------
--------------------------------------------------------------------------------
header_gen <= header_gen_sig;
data_gen <= data_gen_sig;
selBlock <= selBlock_sig;
fin_gen <= fin_gen_sig;
valid <= valid_sig;

end Behavioral;
