----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 10.04.2019 16:19:42
-- Design Name: 
-- Module Name: payload_data_generator - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity payload_data_generator is
    Port ( clk : in STD_LOGIC;
           enable_generate : in STD_LOGIC;
           mData : in STD_LOGIC_VECTOR (7 downto 0);
           mLast : in STD_LOGIC;
           payload : out STD_LOGIC_VECTOR (7 downto 0);
           ended_header : out STD_LOGIC;
           ended_data : out STD_LOGIC);
end payload_data_generator;

architecture Behavioral of payload_data_generator is

    --seales internas para entradas
    signal clk_sig :  STD_LOGIC := '0';
    signal enable_generate_sig :  STD_LOGIC := '0';
    signal mData_sig :  STD_LOGIC_VECTOR(7 downto 0) := "00000000";
    signal mLast_sig : STD_LOGIC:= '0';
    --seales internas para salidas
    signal payload_sig : STD_LOGIC_VECTOR (7 downto 0) := "00000000";
    signal ended_header_sig :  STD_LOGIC := '0';
    signal ended_data_sig :  STD_LOGIC := '0';
    
    --seales relativas a la generacion del payload
    signal elemento_generado_sig : integer range 1 to 4 := 1;
    signal frag_MAC_generado_sig : integer range 0 to 6 := 0;
    signal numero_octetos_generados : integer range 0 to 1500 := 0;
    signal suficientes_octetos : STD_LOGIC := '0';
    signal seguimos_generando_payload : STD_LOGIC := '0';
    signal primer_octeto_ethertype_ok : STD_LOGIC := '0';
    -- constantes para generacion de payload
    constant unidad_entero_0to6 : integer range 0 to 6 := 1;
    constant unidad_entero_0to1_5k : integer range 0 to 1500 := 1;
    constant elemento_nada      : integer range 0 to 4 := 0;
    constant elemento_MAC_Dst   : integer range 0 to 4 := 1;
    constant elemento_MAC_Src   : integer range 0 to 4 := 2;
    constant elemento_ethertype : integer range 0 to 4 := 3; 
    constant elemento_payload   : integer range 0 to 4 := 4;    
    constant no_val             : STD_LOGIC_VECTOR (7 downto 0)   := "00000000";
    constant ethertype_val_0    : STD_LOGIC_VECTOR  (7 downto 0)  :=  "00010000"; -- IP Protocol 0x0800
    constant ethertype_val_1    : STD_LOGIC_VECTOR  (7 downto 0)  :=  "00000000";
    constant numero_minimo_octetos : integer range 0 to 1500 := 60; --teniendo en cuenta: MAC_DST(6)+MAC_SRC(6)+ETHERTYPE(2)+DATA(46)
    --MAC destino hh:hh:hh:hh:hh:hh -> bbbbbbbb:bbbbbbbb:bbbbbbbb:bbbbbbbb:bbbbbbbb:bbbbbbbb
    --             0: 1: 2: 3: 4: 5         
    constant MAC_Dst_0 :    STD_LOGIC_VECTOR  (7 downto 0)  :=  "11111111";
    constant MAC_Dst_1 :    STD_LOGIC_VECTOR  (7 downto 0)  :=  "11111111";
    constant MAC_Dst_2 :    STD_LOGIC_VECTOR  (7 downto 0)  :=  "11111111";
    constant MAC_Dst_3 :    STD_LOGIC_VECTOR  (7 downto 0)  :=  "11111111";
    constant MAC_Dst_4 :    STD_LOGIC_VECTOR  (7 downto 0)  :=  "11111111";
    constant MAC_Dst_5 :    STD_LOGIC_VECTOR  (7 downto 0)  :=  "11111111";
    --MAC origen hh:hh:hh:hh:hh:hh -> bbbbbbbb:bbbbbbbb:bbbbbbbb:bbbbbbbb:bbbbbbbb:bbbbbbbb
    --            0: 1: 2: 3: 4: 5 
    constant MAC_Src_0 :    STD_LOGIC_VECTOR  (7 downto 0)  :=  "11010101";--ab
    constant MAC_Src_1 :    STD_LOGIC_VECTOR  (7 downto 0)  :=  "00111101";--bc
    constant MAC_Src_2 :    STD_LOGIC_VECTOR  (7 downto 0)  :=  "10110011";--cd
    constant MAC_Src_3 :    STD_LOGIC_VECTOR  (7 downto 0)  :=  "01111011";--de
    constant MAC_Src_4 :    STD_LOGIC_VECTOR  (7 downto 0)  :=  "11110111";--ef
    constant MAC_Src_5 :    STD_LOGIC_VECTOR  (7 downto 0)  :=  "01011111";--fa
    
begin

    process (clk_sig)
    begin
         if (clk_sig'event and clk_sig='1') then
            if (enable_generate_sig = '1') then
            -- solamente generamos cuando el bloque de control lo indique
            
            -- para cada elemento a generar se establece el dato de salida y el siguiente elemento a generar
--                if (elemento_generado_sig = 0) then
--                -- comienza la generacion del payload del paquete
--                    elemento_generado_sig <= 1;
--                    frag_MAC_generado_sig <= 0;
--                els
                if (elemento_generado_sig = elemento_MAC_Dst) then
                -- se genera la MAC Destino
                    frag_MAC_generado_sig <= (frag_MAC_generado_sig + unidad_entero_0to6);
                    
                    if (frag_MAC_generado_sig = 1) then 
                        payload_sig <= MAC_Dst_0; 
                    elsif (frag_MAC_generado_sig = 2) then 
                        payload_sig <= MAC_Dst_1;
                    elsif (frag_MAC_generado_sig = 3) then 
                        payload_sig <= MAC_Dst_2;                       
                    elsif (frag_MAC_generado_sig = 4) then
                        payload_sig <= MAC_Dst_3;
                    elsif (frag_MAC_generado_sig = 5) then
                        payload_sig <= MAC_Dst_4;
                    elsif (frag_MAC_generado_sig = 6) then
                        payload_sig <= MAC_Dst_5;
                        elemento_generado_sig <= 2;
                        frag_MAC_generado_sig <= 0;
                    end if;


                                    
                elsif (elemento_generado_sig = elemento_MAC_Src) then
                -- se genera la MAC Source
                    frag_MAC_generado_sig <= frag_MAC_generado_sig + unidad_entero_0to6;
                                        
                    if (frag_MAC_generado_sig = 1) then 
                        payload_sig <= MAC_Src_0; 
                    elsif (frag_MAC_generado_sig = 2) then 
                        payload_sig <= MAC_Src_1;
                    elsif (frag_MAC_generado_sig = 3) then 
                        payload_sig <= MAC_Src_2;                       
                    elsif (frag_MAC_generado_sig = 4) then
                        payload_sig <= MAC_Src_3;
                    elsif (frag_MAC_generado_sig = 5) then
                        payload_sig <= MAC_Src_4;
                    elsif (frag_MAC_generado_sig = 6) then
                        payload_sig <= MAC_Src_5;
                        elemento_generado_sig <= 3;
                        frag_MAC_generado_sig <= 0;
                        numero_octetos_generados <= 12;
                        --ended_header_sig <= '1';
                        --seguimos_generando_payload <= '0';
                        primer_octeto_ethertype_ok <= '0';
                    end if;
                    
                elsif (elemento_generado_sig = elemento_ethertype) then
                -- se genera el campo Ethertype
                    if (primer_octeto_ethertype_ok = '0') then
                        primer_octeto_ethertype_ok <= '1';
                        payload_sig <= ethertype_val_0;        
                        numero_octetos_generados <= numero_octetos_generados + 1;  
                    else
                        payload_sig <= ethertype_val_1;   
                        elemento_generado_sig <= 4;
                        frag_MAC_generado_sig <= 0;     
                        numero_octetos_generados <= numero_octetos_generados + 1; 
                        ended_header_sig <= '1';
                        seguimos_generando_payload <= '0'; 
                    end if;
                          
                elsif (elemento_generado_sig = elemento_payload) then
                -- se genera la Informacion (payload)
                
                    numero_octetos_generados <= numero_octetos_generados + unidad_entero_0to1_5k; --incremento del numero de octetos generados
                    if (seguimos_generando_payload = '0') then
                        --funcionamiento normal: datos procedentes del generador
                        payload_sig <= mData_sig;             
                        
                        if (mLast_sig = '1') then
                            --si generador llega a final de cuenta de generacion de datos
                            if (numero_octetos_generados = numero_minimo_octetos OR numero_octetos_generados > numero_minimo_octetos)then
                                --la trama tiene los octetos suficientes y se detiene la generacion de payload
                                ended_data_sig <= '1';
                            else
                                --es necesario introducir datos nulos hasta tamao de trama minimo
                                seguimos_generando_payload <= '1';
                            end if;
                        end if;
                                       
                    else
                        --funcionamiento auxiliar: datos adicionales nulos para rellenar trama hasta tamao minimo
                        payload_sig <= no_val;
                        
                        if (numero_octetos_generados = numero_minimo_octetos OR numero_octetos_generados > numero_minimo_octetos)then
                            --la trama tiene los octetos suficientes y se detiene la generacion de payload
                            ended_data_sig <= '1';
                        end if;
                        
                    end if;
        
                else
                    payload_sig <= no_val;
                    elemento_generado_sig <= 1;
                    frag_MAC_generado_sig <= 1;
                end if;
            else
                -- si no esta habilitado el bloque la salida sera nula
                elemento_generado_sig <= 1;
                frag_MAC_generado_sig <= 1;
                ended_header_sig <= '0';
                ended_data_sig <= '0';
                payload_sig <= no_val;
                numero_octetos_generados <= 0;
                seguimos_generando_payload <= '0';
            end if; 
        end if; 
             
    end process;

--Asignacion de entradas a seales internas
clk_sig <= clk;
enable_generate_sig <= enable_generate;
mData_sig <= mData;
mLast_sig <= mLast;
--Asignacion de seales internas a salidas
payload <= payload_sig;
ended_header <= ended_header_sig;
ended_data <= ended_data_sig;

end Behavioral;
